'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const client = require('@onomy/client');
const reactUtils = require('@onomy/react-utils');
const hub = require('@onomy/hub');
const reactHub = require('@onomy/react-hub');
const BigNumber = require('bignumber.js');
const React = require('react');
const client$1 = require('@apollo/client');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

const BigNumber__default = /*#__PURE__*/_interopDefaultLegacy(BigNumber);
const React__default = /*#__PURE__*/_interopDefaultLegacy(React);

const onomyVestingSchedules = [
  { amount: new BigNumber__default["default"](29367338), duration: 36, cliff: 12 },
  { amount: new BigNumber__default["default"](3595397), duration: 30, cliff: 12 },
  { amount: new BigNumber__default["default"](12302874), duration: 24, cliff: 12 }
];
const DEVNET = {
  onomyChainInfo: client.OnomyConstants.CHAIN_INFO.TESTNET,
  onomyVestingSchedules,
  onomyStartDate: new Date("2022-11-29"),
  onomyDaoAddress: "onomy1vwr8z00ty7mqnk4dtchr9mn9j96nuh6wrlww93",
  onomyGraphqlEndpoint: "https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-devnet",
  onexChainInfo: client.OnexConstants.CHAIN_INFO.DEVNET,
  onexGraphqlEndpoint: "https://graphql-onex-devnet.onomy.io/subgraphs/name/onomyprotocol/mgraph",
  ethNetworkName: "goerli",
  ethWsUrl: "",
  bNomContractAddress: "0x8EFe26D6839108E831D3a37cA503eA4F136A8E73",
  ethNomContractAddress: "0x9653a2013e23ae976d34543572e3e72f8578e4c6",
  bondContractAddress: "0xD179c5BEd30cADE4E62d53DD89240745Fb4C0CC2",
  gravityContractAddress: "0x18619DE15bDd14b0360e82e2746aAf77B17b3925",
  wEthUsdUniswapPoolAddress: "",
  ethBridgeWaitBlocks: 35
};
const TESTNET = {
  onomyChainInfo: client.OnomyConstants.CHAIN_INFO.TESTNET,
  onomyVestingSchedules,
  onomyStartDate: new Date("2022-11-29"),
  onomyDaoAddress: "onomy1vwr8z00ty7mqnk4dtchr9mn9j96nuh6wrlww93",
  onomyGraphqlEndpoint: "https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-testnet",
  onexChainInfo: client.OnexConstants.CHAIN_INFO.TESTNET,
  onexGraphqlEndpoint: "https://graphql-onex-testnet.onomy.io/subgraphs/name/onomyprotocol/mgraph",
  ethNetworkName: "goerli",
  ethWsUrl: "",
  bNomContractAddress: "0x8EFe26D6839108E831D3a37cA503eA4F136A8E73",
  ethNomContractAddress: "",
  bondContractAddress: "0xD179c5BEd30cADE4E62d53DD89240745Fb4C0CC2",
  gravityContractAddress: "0xBA7cDCD837757EB67Ea889d168Aba1a1Fa7C1340",
  wEthUsdUniswapPoolAddress: "",
  ethBridgeWaitBlocks: 64
};
const MAINNET = {
  onomyChainInfo: client.OnomyConstants.CHAIN_INFO.MAINNET,
  onomyVestingSchedules,
  onomyStartDate: new Date("2022-11-29"),
  onomyDaoAddress: "onomy1vwr8z00ty7mqnk4dtchr9mn9j96nuh6wrlww93",
  onomyGraphqlEndpoint: "https://api.thegraph.com/subgraphs/name/onomyprotocol/ograph-mainnet",
  onexChainInfo: void 0,
  onexGraphqlEndpoint: "",
  ethNetworkName: "homestead",
  ethWsUrl: "",
  bNomContractAddress: "0x4fdF157C6860F33232608198419c74ED446Ca577",
  ethNomContractAddress: "",
  bondContractAddress: "0x264C13cfEd981e3137Fb43B198D14D8D5D64977E",
  gravityContractAddress: "0x85391fd149282C50B103aC810430F887685d575C",
  wEthUsdUniswapPoolAddress: "0x4e68ccd3e89f51c3074ca5072bbac773960dfa36",
  ethBridgeWaitBlocks: 64
};
const OnomyEcosystemConfigs = {
  DEVNET,
  TESTNET,
  MAINNET
};
function setOnomyEcosystemConfig(configName, overrides) {
  const config = { ...OnomyEcosystemConfigs[configName], ...overrides != null ? overrides : {} };
  hub.OnomyHubConstants.updateAssetData({
    bNomAsset: {
      id: config.bNomContractAddress
    },
    ethNomAsset: {
      id: config.ethNomContractAddress
    },
    ethChainId: config.ethNetworkName
  });
  return config;
}

function useOnexClient() {
  var _a;
  const { hub } = reactHub.useOnomyHub();
  const onex = (_a = hub == null ? void 0 : hub.chains.onex) == null ? void 0 : _a.client;
  if (!onex)
    return null;
  return onex;
}
function useOnexBurned() {
  var _a;
  const onex = useOnexClient();
  const chain = onex == null ? void 0 : onex.getBlockchain();
  const [assets, assetsStatus] = reactHub.useSupportedAssetList();
  const [burnedRaw, burnedStatus] = reactUtils.useAsyncPoll(React.useCallback(async () => {
    var _a2;
    return (_a2 = onex == null ? void 0 : onex.burned()) != null ? _a2 : null;
  }, [onex]), null);
  const asset = React.useMemo(() => {
    if (!burnedRaw || !(chain == null ? void 0 : chain.name))
      return hub.OnomyHubConstants.UNKNOWN_BALANCE.asset;
    const burnAsset = assets.find((a) => a.id === burnedRaw.denom && a.chain.name === chain.name);
    return burnAsset != null ? burnAsset : hub.OnomyHubConstants.UNKNOWN_BALANCE.asset;
  }, [assets, burnedRaw, chain == null ? void 0 : chain.name]);
  const [amount, amountStatus] = reactHub.useDetailedBalance(asset, (_a = burnedRaw == null ? void 0 : burnedRaw.amount) != null ? _a : "0");
  const status = reactUtils.useAsyncStatuses([assetsStatus, burnedStatus, amountStatus]);
  return [amount, status];
}
function useOnexPool({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const pair = React.useMemo(() => [baseDenom, quoteDenom].sort().join(","), [baseDenom, quoteDenom]);
  const cb = React.useCallback(async () => {
    var _a, _b;
    try {
      const res = (_a = await (onex == null ? void 0 : onex.pool(pair))) != null ? _a : null;
      if (!res || res.drops === "0")
        return null;
      return res;
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return null;
      throw e;
    }
  }, [pair, onex]);
  return reactUtils.useAsyncPoll(cb, null);
}
function useOnexPairMembers({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const cb = React.useCallback(async () => {
    var _a, _b, _c;
    try {
      const [memberBase, memberQuote] = await Promise.all([
        (_a = onex == null ? void 0 : onex.member({
          denomA: quoteDenom,
          denomB: baseDenom
        })) != null ? _a : Promise.resolve(null),
        (_b = onex == null ? void 0 : onex.member({
          denomA: baseDenom,
          denomB: quoteDenom
        })) != null ? _b : Promise.resolve(null)
      ]);
      return { memberBase, memberQuote };
    } catch (e) {
      if ((_c = e == null ? void 0 : e.message) == null ? void 0 : _c.match(/not found/))
        return { memberBase: null, memberQuote: null };
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);
  return reactUtils.useAsyncPoll(cb, { memberBase: null, memberQuote: null });
}
async function includePoolDetails({
  hub,
  onex,
  baseAsset,
  quoteAsset,
  pool,
  memberBase,
  memberQuote
}) {
  if (!hub)
    throw new Error("Hub is null");
  const [memberBaseTotal, memberQuoteTotal, leaders] = await Promise.all([
    hub.balances.includeDetails({
      amount: BigInt(memberBase.balance),
      asset: baseAsset
    }),
    hub.balances.includeDetails({
      amount: BigInt(memberQuote.balance),
      asset: quoteAsset
    }),
    Promise.all(pool.leaders.map(async (leader) => {
      const amounts = await (async () => {
        var _a;
        const fallback = {
          denom1: baseAsset.id,
          denom2: quoteAsset.id,
          amount1: "0",
          amount2: "0"
        };
        try {
          return (_a = await (onex == null ? void 0 : onex.dropsToCoins(baseAsset.id, quoteAsset.id, leader.drops))) != null ? _a : fallback;
        } catch (e) {
          console.warn("Error fetching dropsToCoins", { pair: pool.pair, drops: leader.drops }, e);
          return fallback;
        }
      })();
      const leaderBaseAmountFixed = amounts.amount1;
      const leaderQuoteAmountFixed = amounts.amount2;
      const [baseAmount, quoteAmount] = await Promise.all([
        hub.balances.includeDetails({
          amount: BigInt(leaderBaseAmountFixed),
          asset: baseAsset
        }),
        hub.balances.includeDetails({
          amount: BigInt(leaderQuoteAmountFixed),
          asset: quoteAsset
        })
      ]);
      return {
        ...leader,
        baseAmount,
        quoteAmount
      };
    }))
  ]);
  return {
    ...pool,
    baseAsset,
    quoteAsset,
    fiatTotal: memberBaseTotal.fiatValue + memberQuoteTotal.fiatValue,
    fiatCurrency: memberBaseTotal.fiatCurrency,
    leaders,
    memberBase: {
      ...memberBase,
      total: memberBaseTotal
    },
    memberQuote: {
      ...memberQuote,
      total: memberQuoteTotal
    }
  };
}
function useOnexPoolDetail({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const { hub } = reactHub.useOnomyHub();
  const onex = useOnexClient();
  const [pool, poolStatus] = useOnexPool({ baseDenom, quoteDenom });
  const [{ memberBase, memberQuote }, membersStatus] = useOnexPairMembers({
    baseDenom,
    quoteDenom
  });
  const detailedPoolCb = React.useCallback(async () => {
    if (!pool || !memberBase || !memberQuote || !hub)
      return null;
    return includePoolDetails({ hub, onex, baseAsset, quoteAsset, pool, memberBase, memberQuote });
  }, [hub, onex, baseAsset, quoteAsset, pool, memberBase, memberQuote]);
  const [detailedPool, detailedPoolStatus] = reactUtils.useAsyncPoll(detailedPoolCb, null);
  const combinedStatus = reactUtils.useAsyncStatuses([poolStatus, membersStatus, detailedPoolStatus]);
  return [detailedPool, combinedStatus];
}
function useOnexPoolPrice({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const cb = React.useCallback(async () => {
    var _a, _b;
    try {
      return (_a = await (onex == null ? void 0 : onex.getPoolDisplayPrice({ baseDenom, quoteDenom }))) != null ? _a : "0";
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return "0";
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);
  return reactUtils.useAsyncPoll(cb, "0");
}
function useFixedAmountFromDisplay(displayValue, asset) {
  return React.useMemo(() => {
    const bn = new BigNumber__default["default"](displayValue);
    if (bn.isNaN())
      return "0";
    return bn.shiftedBy(asset.meta.decimalPlaces).toFixed(0);
  }, [displayValue, asset.meta.decimalPlaces]);
}
function useOnexDetailedPoolPrice({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = reactHub.useDetailedBalance(quoteAsset, currentPriceFixed);
  const status = reactUtils.useAsyncStatuses([currentPriceStatus, currentPricePerBaseStatus]);
  return [currentPricePerBase, status];
}
function useOnexLastExecutionPrice({ baseDenom = "anom", quoteDenom = "usdt" }) {
  const onex = useOnexClient();
  const cb = React.useCallback(async () => {
    var _a, _b;
    try {
      return (_a = await (onex == null ? void 0 : onex.getLastTradeDisplayPrice({ baseDenom, quoteDenom }))) != null ? _a : "0";
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return "0";
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);
  return reactUtils.useAsyncPoll(cb, "0");
}
function useOnexDetailedLastExecutionPrice({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const [currentPrice, currentPriceStatus] = useOnexLastExecutionPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = reactHub.useDetailedBalance(quoteAsset, currentPriceFixed);
  const status = reactUtils.useAsyncStatuses([currentPriceStatus, currentPricePerBaseStatus]);
  return [currentPricePerBase, status];
}
function useAsset(id, chain) {
  var _a;
  const [assets, status] = reactHub.useSupportedAssetList();
  const asset = (_a = assets.find((a) => {
    return a.id === id && (chain == null ? void 0 : chain.name) === a.chain.name;
  })) != null ? _a : {
    ...hub.OnomyHubConstants.UNKNOWN_BALANCE.asset,
    chain
  };
  return [asset, status];
}
async function includeOrderDetail({
  hub,
  onex,
  order,
  baseAsset,
  quoteAsset
}) {
  var _a;
  if (!hub)
    throw new Error("Hub is null");
  const direction = order.denomAsk === baseAsset.id ? "buy" : "sell";
  const baseAmount = {
    amount: direction === "sell" ? BigInt(order.amount) : BigInt(new BigNumber__default["default"](order.rate[0]).times(order.amount).dividedBy(order.rate[1]).toFixed(0)),
    asset: baseAsset
  };
  const quoteAmountSimple = {
    amount: direction === "buy" ? BigInt(order.amount) : BigInt(new BigNumber__default["default"](order.rate[0]).times(order.amount).dividedBy(order.rate[1]).toFixed(0)),
    asset: quoteAsset
  };
  const [price, amount, quoteAmount] = await Promise.all([
    (_a = onex == null ? void 0 : onex.getDisplayPriceFromRatio({
      baseDenom: baseAsset.id,
      baseAmountFixed: order.rate[direction === "buy" ? 0 : 1],
      quoteDenom: quoteAsset.id,
      quoteAmountFixed: order.rate[direction === "buy" ? 1 : 0]
    })) != null ? _a : Promise.resolve("0"),
    hub.balances.includeDetails(baseAmount),
    hub.balances.includeDetails(quoteAmountSimple)
  ]);
  const bnPrice = new BigNumber__default["default"](price).shiftedBy(quoteAsset.meta.decimalPlaces);
  const pricePer = await hub.balances.includeDetails({
    amount: BigInt(bnPrice.toFixed(0)),
    asset: quoteAsset
  });
  return {
    ...order,
    amount,
    quoteAmount,
    price,
    pricePer,
    direction,
    baseAsset,
    quoteAsset
  };
}
function useOnexOrderHistory({
  baseAsset,
  quoteAsset,
  length = 10
}) {
  const { hub } = reactHub.useOnomyHub();
  const onex = useOnexClient();
  const pair = React.useMemo(() => [baseAsset.id, quoteAsset.id].sort().join(","), [baseAsset.id, quoteAsset.id]);
  const cb = React.useCallback(async () => {
    var _a;
    try {
      const orders = hub && onex ? await onex.history({ pair, length: `${length}` }) : [];
      return await Promise.all(orders.map((order) => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order })));
    } catch (e) {
      if ((_a = e == null ? void 0 : e.message) == null ? void 0 : _a.match(/not found/))
        return [];
      throw e;
    }
  }, [baseAsset.id, quoteAsset.id, length, pair, hub, onex]);
  return reactUtils.useAsyncPoll(cb, []);
}
function useOnexMyActiveOrders({
  baseAsset,
  quoteAsset,
  length = 10
}) {
  const { hub } = reactHub.useOnomyHub();
  const onex = useOnexClient();
  const pair = React.useMemo(() => [baseAsset.id, quoteAsset.id].sort().join(","), [baseAsset.id, quoteAsset.id]);
  const cb = React.useCallback(async () => {
    const orders = hub && onex ? await onex.ordersByOwner() : [];
    return Promise.all(orders.filter((order) => [order.denomAsk, order.denomBid].sort().join(",") === pair).map((order) => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order })));
  }, [baseAsset.id, quoteAsset.id, pair, length, hub, onex]);
  return reactUtils.useAsyncPoll(cb, []);
}
function useOnexMarketEstimate({ denomAsk = "usdt", denomBid = "anom", amountBid = "0" }) {
  const onex = useOnexClient();
  const cb = React.useCallback(async () => {
    const bnAmount = new BigNumber__default["default"](amountBid);
    if (!onex || bnAmount.lte(0) || bnAmount.isNaN() || !bnAmount.isFinite())
      return "0";
    const resp = await onex.quote({
      denomAsk,
      denomBid,
      denomAmount: denomBid,
      amount: amountBid
    });
    return resp.amount;
  }, [denomAsk, denomBid, amountBid, onex]);
  return reactUtils.useAsyncPoll(cb, "0");
}
function useOnexOrderBook({
  baseAsset,
  quoteAsset,
  direction,
  orderType = "limit"
}) {
  const { hub } = reactHub.useOnomyHub();
  const onex = useOnexClient();
  const cb = React.useCallback(async () => {
    const denomA = direction === "buy" ? baseAsset.id : quoteAsset.id;
    const denomB = direction === "sell" ? baseAsset.id : quoteAsset.id;
    const book = hub && onex ? await onex.book({ denomA, denomB, orderType }) : [];
    const res = await Promise.all(book.map((order) => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order })));
    return direction === "buy" ? res : res.reverse();
  }, [onex, baseAsset, quoteAsset, direction, hub, orderType]);
  return reactUtils.useAsyncPoll(cb, []);
}
function calculateDrops(pool, rawDrops) {
  const bnBaseMemberAmount = new BigNumber__default["default"](pool.memberBase.balance);
  const bnQuoteMemberAmount = new BigNumber__default["default"](pool.memberQuote.balance);
  const bnPoolDrops = new BigNumber__default["default"](pool.drops);
  return rawDrops.map((drop) => {
    const bnProduct = new BigNumber__default["default"](drop.product);
    const bnDrops = new BigNumber__default["default"](drop.drops);
    const bnTotalBase = bnBaseMemberAmount.multipliedBy(bnDrops).dividedBy(bnPoolDrops);
    const bnTotalQuote = bnQuoteMemberAmount.multipliedBy(bnDrops).dividedBy(bnPoolDrops);
    const bnPrincipalBase = bnProduct.multipliedBy(bnBaseMemberAmount).dividedBy(bnQuoteMemberAmount).sqrt();
    const bnPrincipalQuote = bnProduct.multipliedBy(bnQuoteMemberAmount).dividedBy(bnBaseMemberAmount).sqrt();
    const bnRewardBase = bnTotalBase.minus(bnPrincipalBase);
    const bnRewardQuote = bnTotalQuote.minus(bnPrincipalQuote);
    return {
      ...drop,
      bnTotalBase,
      bnTotalQuote,
      bnPrincipalBase,
      bnPrincipalQuote,
      bnRewardBase,
      bnRewardQuote
    };
  });
}
async function includeDropsDetails(hub, pool, calculatedDrops) {
  if (!hub)
    throw new Error("Hub is null");
  return Promise.all(calculatedDrops.map(async (drop) => {
    const [totalBase, totalQuote, principalBase, principalQuote, rewardBase, rewardQuote] = await Promise.all([
      hub.balances.includeDetails({
        amount: BigInt(drop.bnTotalBase.toFixed(0)),
        asset: pool.baseAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnTotalQuote.toFixed(0)),
        asset: pool.quoteAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnPrincipalBase.toFixed(0)),
        asset: pool.baseAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnPrincipalQuote.toFixed(0)),
        asset: pool.quoteAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnRewardBase.toFixed(0)),
        asset: pool.baseAsset
      }),
      hub.balances.includeDetails({
        amount: BigInt(drop.bnRewardQuote.toFixed(0)),
        asset: pool.quoteAsset
      })
    ]);
    const principalFiat = principalBase.fiatValue + principalQuote.fiatValue;
    const rewardFiat = rewardBase.fiatValue + rewardQuote.fiatValue;
    const totalFiat = principalFiat + rewardFiat;
    return {
      ...drop,
      pool,
      totalBase,
      totalQuote,
      principalBase,
      principalQuote,
      rewardBase,
      rewardQuote,
      principalFiat,
      rewardFiat,
      totalFiat
    };
  }));
}
function useOnexMyDropsForPool(pool) {
  const { hub } = reactHub.useOnomyHub();
  const onex = useOnexClient();
  const rawDropsCb = React.useCallback(async () => {
    var _a;
    if (!onex || !(pool == null ? void 0 : pool.pair))
      return [];
    try {
      return await onex.dropOwnerPair(pool.pair);
    } catch (e) {
      if ((_a = e == null ? void 0 : e.message) == null ? void 0 : _a.match(/not found/))
        return [];
      throw e;
    }
  }, [onex, pool == null ? void 0 : pool.pair]);
  const [rawDrops, rawDropsStatus] = reactUtils.useAsyncPoll(rawDropsCb, []);
  const calculatedDrops = React.useMemo(() => {
    if (!(pool == null ? void 0 : pool.pair))
      return [];
    return calculateDrops(pool, rawDrops);
  }, [rawDrops, pool == null ? void 0 : pool.pair, pool == null ? void 0 : pool.drops, pool == null ? void 0 : pool.memberBase.balance, pool == null ? void 0 : pool.memberQuote.balance]);
  const detailedDropsCb = React.useCallback(async () => {
    if (!hub || !pool)
      return [];
    return includeDropsDetails(hub, pool, calculatedDrops);
  }, [hub, calculatedDrops]);
  const [detailedDrops, detailedDropsStatus] = reactUtils.useAsyncPoll(detailedDropsCb, []);
  const status = reactUtils.useAsyncStatuses([rawDropsStatus, detailedDropsStatus]);
  return [detailedDrops, status];
}
function useOnexMyDrops() {
  const { hub: hub$1 } = reactHub.useOnomyHub();
  const onex = useOnexClient();
  const dropPairsCb = React.useCallback(async () => {
    var _a, _b;
    try {
      return (_a = await (onex == null ? void 0 : onex.dropPairs())) != null ? _a : [];
    } catch (e) {
      if ((_b = e == null ? void 0 : e.message) == null ? void 0 : _b.match(/not found/))
        return [];
      throw e;
    }
  }, [onex]);
  const [allAssets, assetsStatus] = reactHub.useSupportedAssetList();
  const [rawPairs, rawPairsStatus] = reactUtils.useAsyncPoll(dropPairsCb, []);
  const allDropsCb = React.useCallback(async () => {
    if (!onex)
      return [];
    const onexChain = onex.getBlockchain();
    const onexAssets = allAssets.filter((asset) => asset.chain.name === onexChain.name);
    if (!hub$1 || onexAssets.length <= 0)
      return [];
    return Promise.all(rawPairs.map(async (pair) => {
      const [baseDenom, quoteDenom] = pair.split(",");
      const baseAsset = onexAssets.find((asset) => asset.id === baseDenom);
      const quoteAsset = onexAssets.find((asset) => asset.id === quoteDenom);
      if (!baseAsset)
        throw new Error(`baseAsset not found for ${baseDenom}`);
      if (!quoteAsset)
        throw new Error(`quoteAsset not found for ${quoteDenom}`);
      const [rawDrops, pool, memberBase, memberQuote] = await Promise.all([
        onex.dropOwnerPair(pair),
        onex.pool(pair),
        onex.member({ denomA: quoteDenom, denomB: baseDenom }),
        onex.member({ denomA: baseDenom, denomB: quoteDenom })
      ]);
      const detailedPool = await includePoolDetails({
        hub: hub$1,
        onex,
        baseAsset,
        quoteAsset,
        pool,
        memberBase,
        memberQuote
      });
      const calculatedDrops = await calculateDrops(detailedPool, rawDrops);
      const drops = await includeDropsDetails(hub$1, detailedPool, calculatedDrops);
      const basePrincipalAmounts = drops.map((drop) => drop.principalBase);
      const quotePrincipalAmounts = drops.map((drop) => drop.principalQuote);
      const baseRewardAmounts = drops.map((drop) => drop.rewardBase);
      const quoteRewardAmounts = drops.map((drop) => drop.rewardQuote);
      const [basePrincipalRaw] = hub.OnomyHubUtils.combineBalances(basePrincipalAmounts);
      const [quotePrincipalRaw] = hub.OnomyHubUtils.combineBalances(quotePrincipalAmounts);
      const [baseRewardRaw] = hub.OnomyHubUtils.combineBalances(baseRewardAmounts);
      const [quoteRewardRaw] = hub.OnomyHubUtils.combineBalances(quoteRewardAmounts);
      const [baseTotalRaw] = hub.OnomyHubUtils.combineBalances([basePrincipalRaw, baseRewardRaw]);
      const [quoteTotalRaw] = hub.OnomyHubUtils.combineBalances([quotePrincipalRaw, quoteRewardRaw]);
      const [basePrincipal, quotePrincipal, baseReward, quoteReward, baseTotal, quoteTotal] = await Promise.all([
        hub$1.balances.includeDetails(basePrincipalRaw),
        hub$1.balances.includeDetails(quotePrincipalRaw),
        hub$1.balances.includeDetails(baseRewardRaw),
        hub$1.balances.includeDetails(quoteRewardRaw),
        hub$1.balances.includeDetails(baseTotalRaw),
        hub$1.balances.includeDetails(quoteTotalRaw)
      ]);
      const fiatPrincipal = basePrincipal.fiatValue + quotePrincipal.fiatValue;
      const fiatReward = baseReward.fiatValue + quoteReward.fiatValue;
      const fiatTotal = fiatPrincipal + fiatReward;
      return {
        pool: detailedPool,
        drops,
        basePrincipal,
        quotePrincipal,
        baseReward,
        quoteReward,
        baseTotal,
        quoteTotal,
        fiatPrincipal,
        fiatReward,
        fiatTotal
      };
    }));
  }, [hub$1, onex, rawPairs, allAssets]);
  return reactUtils.useAsyncPoll(allDropsCb, []);
}

function useExchangePairs() {
  const onex = useOnexClient();
  const [allAssets, assetsStatus] = reactHub.useSupportedAssetList();
  const onexAssets = React.useMemo(() => {
    var _a;
    const chainId = (_a = onex == null ? void 0 : onex.getBlockchain().name) != null ? _a : "";
    const res = allAssets.filter((asset) => {
      return asset.chain.name === chainId && asset.id !== "anom" && asset.id !== "onex";
    });
    const anom = allAssets.find((asset) => {
      return asset.chain.name === chainId && asset.id === "anom";
    });
    return anom ? [anom, ...res] : res;
  }, [allAssets, onex]);
  const quoteAssets = onexAssets;
  const pairsByQuote = React.useMemo(() => {
    return onexAssets.reduce((res, quoteAsset) => {
      return {
        ...res,
        [quoteAsset.id]: {
          quoteAsset,
          pairs: onexAssets.filter((asset) => asset.id !== quoteAsset.id).map((baseAsset) => {
            return {
              quoteAsset,
              baseAsset,
              pair: [quoteAsset.id, baseAsset.id].sort().join(",")
            };
          })
        }
      };
    }, {});
  }, [onexAssets]);
  const loadingStatus = reactUtils.useAsyncStatuses([assetsStatus]);
  return {
    loadingStatus,
    assetsStatus,
    onexAssets,
    quoteAssets,
    pairsByQuote
  };
}

const BLANK_ONEX_HISTORY_FRAME = {
  startTime: "0",
  endTime: "0",
  startPrice: "0",
  maxPrice: "0",
  minPrice: "0",
  endPrice: "0",
  volume: "0"
};
const ONEX_HISTORICAL_DATA_QUERY = client$1.gql`
  query historicalFrames(
    $baseDenom: String!
    $quoteDenom: String!
    $frameType: String!
    $skipped: Int!
    $first: Int
    $from: Int
    $to: Int
  ) {
    historicalFrames(
      first: $first
      skip: $skipped
      where: { base: $baseDenom, quote: $quoteDenom, type: $frameType, startTime_lte: $to }
      orderBy: startTime
      orderDirection: desc
    ) {
      startTime
      endTime
      startPrice
      endPrice
      minPrice
      maxPrice
      volume
    }
  }
`;
class OnexHistoryFrames extends hub.AbstractThrottledAsyncCache {
  constructor(apolloClient) {
    super();
    this.apolloClient = apolloClient;
  }
  cacheKey([{ baseDenom, quoteDenom, frameType, from, to, first, skipped }]) {
    return [baseDenom, quoteDenom, frameType, from, to, first, skipped].join("-");
  }
  async implementation({
    baseDenom,
    quoteDenom,
    frameType,
    from,
    to,
    first,
    skipped
  }) {
    try {
      const resp = await this.apolloClient.query({
        query: ONEX_HISTORICAL_DATA_QUERY,
        variables: {
          frameType,
          skipped,
          baseDenom,
          quoteDenom,
          from: from ? Math.round(from) : void 0,
          to: to ? Math.round(to) : Math.round(new Date().getTime() / 1e3),
          first
        }
      });
      const {
        data: { historicalFrames }
      } = resp;
      return historicalFrames;
    } catch (e) {
      console.error("OnexExchangeHistory error", e);
      throw e;
    }
  }
}

function useOnomyAppState(ecosystem) {
  const { wallet } = reactHub.useOnomyHub();
  const [myOnomyAddress, onomyAddressStatus] = reactHub.useAddress(hub.OnomyHubConstants.ONOMY_CHAIN);
  const [myOnexAddress, onexAddressStatus] = reactHub.useAddress(hub.OnomyHubConstants.ONEX_CHAIN);
  const [onomyNomAsset, onomyNomAssetStatus] = useAsset("anom", hub.OnomyHubConstants.ONOMY_CHAIN);
  const [onexNomAsset, onexNomAssetStatus] = useAsset("anom", hub.OnomyHubConstants.ONEX_CHAIN);
  const [onexUsdtAsset, onexUsdtAssetStatus] = useAsset("ausdt", hub.OnomyHubConstants.ONEX_CHAIN);
  const [onexUsdcAsset, onexUsdcAssetStatus] = useAsset("ausdc", hub.OnomyHubConstants.ONEX_CHAIN);
  const [onexEthAsset, onexEthAssetStatus] = useAsset("wei", hub.OnomyHubConstants.ONEX_CHAIN);
  const { loadingStatus: pairsStatus, quoteAssets, pairsByQuote } = useExchangePairs();
  const onexHistoryFrames = React.useMemo(() => {
    if (!ecosystem.onexGraphqlEndpoint)
      return null;
    const apolloClient = new client$1.ApolloClient({
      uri: ecosystem.onexGraphqlEndpoint,
      cache: new client$1.InMemoryCache()
    });
    return new OnexHistoryFrames(apolloClient);
  }, [ecosystem.onexGraphqlEndpoint]);
  const myOnomyWallet = wallet.onomy.connectedWallet;
  const myOnexWallet = wallet.onex.connectedWallet;
  const loadingStatus = reactUtils.useAsyncStatuses([
    pairsStatus,
    onomyAddressStatus,
    onexAddressStatus,
    onomyNomAssetStatus,
    onexNomAssetStatus,
    onexUsdtAssetStatus,
    onexUsdcAssetStatus,
    onexEthAssetStatus
  ]);
  const logout = React.useCallback(async () => {
    await wallet.disconnect();
    window.location.reload();
  }, [wallet]);
  const logoutOnomy = React.useCallback(async () => {
    await wallet.onomy.disconnect();
    window.location.reload();
  }, [wallet.onomy]);
  const logoutOnex = React.useCallback(async () => {
    await wallet.onex.disconnect();
    window.location.reload();
  }, [wallet.onex]);
  return {
    ecosystem,
    loadingStatus,
    onexHistoryFrames,
    onomyChain: hub.OnomyHubConstants.ONOMY_CHAIN,
    onexChain: hub.OnomyHubConstants.ONEX_CHAIN,
    onomyNomAsset,
    onexNomAsset,
    onexUsdtAsset,
    onexUsdcAsset,
    onexEthAsset,
    myOnomyAddress,
    myOnexAddress,
    myOnomyWallet,
    myOnexWallet,
    quoteAssets,
    pairsByQuote,
    logout,
    logoutOnomy,
    logoutOnex
  };
}
const OnomyAppContext = React.createContext(null);
function useOnomyApp() {
  const state = React.useContext(OnomyAppContext);
  if (!state)
    throw new Error("OnomyAppContext missing");
  return state;
}
function OnomyAppProvider({
  children,
  ecosystem
}) {
  const state = useOnomyAppState(ecosystem);
  return /* @__PURE__ */ React__default["default"].createElement(OnomyAppContext.Provider, {
    value: state
  }, children);
}

function OnomyCoreProvider({
  children,
  createBackendFn,
  config = "DEVNET",
  overrides
}) {
  const ecosystem = React.useMemo(() => {
    return setOnomyEcosystemConfig(config, overrides);
  }, []);
  const backend = React.useMemo(createBackendFn, []);
  return /* @__PURE__ */ React__default["default"].createElement(reactHub.OnomyHubProvider, {
    backend,
    onomyChainInfo: ecosystem.onomyChainInfo,
    onexChainInfo: ecosystem.onexChainInfo,
    ethWsUrl: ecosystem.ethWsUrl,
    nomContractAddress: ecosystem.bNomContractAddress,
    bondContractAddress: ecosystem.bondContractAddress,
    gravityContractAddress: ecosystem.gravityContractAddress,
    wEthUsdtUniswapPoolAddress: ecosystem.wEthUsdUniswapPoolAddress
  }, /* @__PURE__ */ React__default["default"].createElement(reactHub.RequireOnomyHub, null, /* @__PURE__ */ React__default["default"].createElement(OnomyAppProvider, {
    ecosystem
  }, children)));
}

function PlaceholderLoadingIndicator({ text = "Loading..." }) {
  return /* @__PURE__ */ React__default["default"].createElement(React__default["default"].Fragment, null, text);
}

function PlaceholderErrorDisplay({ error: errorProp }) {
  React.useEffect(() => console.error("error", errorProp), [errorProp]);
  const error = (errorProp == null ? void 0 : errorProp.message) || errorProp;
  return /* @__PURE__ */ React__default["default"].createElement(React__default["default"].Fragment, null, `${error}`);
}

function useOnexDayTradeFrame({
  baseAsset,
  quoteAsset
}) {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const { onexHistoryFrames } = useOnomyApp();
  const [lastPricePerBase, lastPricePerBaseStatus] = useOnexDetailedLastExecutionPrice({
    baseAsset,
    quoteAsset
  });
  const [[prevFrame = BLANK_ONEX_HISTORY_FRAME, priceFrame = BLANK_ONEX_HISTORY_FRAME], status] = reactUtils.useSubscribedValue(React.useCallback((cb) => {
    if (baseDenom === quoteDenom || !onexHistoryFrames) {
      cb([]);
      return () => {
      };
    }
    return onexHistoryFrames.subscribe([
      {
        frameType: "Day",
        skipped: 0,
        baseDenom,
        quoteDenom,
        first: 2
      }
    ], cb);
  }, [onexHistoryFrames, baseDenom, quoteDenom]), [BLANK_ONEX_HISTORY_FRAME, BLANK_ONEX_HISTORY_FRAME]);
  const startPrice = parseFloat(priceFrame.startPrice);
  const endPrice = parseFloat(priceFrame.endPrice);
  const priceDiff = endPrice - startPrice;
  const priceChangeRatio = priceDiff / startPrice;
  const volumeDiff = new BigNumber__default["default"](priceFrame.volume).minus(prevFrame.volume);
  const volumeDiffRatio = volumeDiff.dividedBy(prevFrame.volume).toNumber();
  const [volumeAmount, volumeAmountStatus] = reactHub.useDetailedBalance(baseAsset, priceFrame.volume);
  const [volumeDiffAmount, volumeDiffAmountStatus] = reactHub.useDetailedBalance(baseAsset, volumeDiff.toFixed(0));
  const loadingStatus = reactUtils.useAsyncStatuses([
    lastPricePerBaseStatus,
    status,
    volumeAmountStatus,
    volumeDiffAmountStatus
  ]);
  return [
    {
      ...priceFrame,
      loadingStatus,
      lastPricePerBase,
      lastPricePerBaseStatus,
      priceDiff,
      priceChangeRatio,
      volumeDiffRatio,
      volumeAmount,
      volumeAmountStatus,
      volumeDiffAmount,
      volumeDiffAmountStatus
    },
    status
  ];
}

function useOnomyDashboardState() {
  const {
    onomyNomAsset,
    onexNomAsset,
    onexEthAsset,
    onexUsdtAsset,
    loadingStatus: appLoadingStatus
  } = useOnomyApp();
  const holdings = reactHub.useHoldings();
  const [myLiquidity, myLiquidityStatus] = useOnexMyDrops();
  const [delegations, delegationsStatus] = reactHub.useDelegations();
  const oneNomFixed = React.useMemo(() => new BigNumber__default["default"]("1").shiftedBy(onomyNomAsset.meta.decimalPlaces).toFixed(0), [onomyNomAsset.meta.decimalPlaces]);
  const [oneNomCoin] = reactHub.useDetailedBalance(onomyNomAsset, oneNomFixed);
  const { principalTotalFiat, rewardTotalFiat, liquidityTotalFiat } = React.useMemo(() => {
    return myLiquidity.reduce((res, item) => {
      res.principalTotalFiat += item.fiatPrincipal;
      res.rewardTotalFiat += item.fiatReward;
      res.liquidityTotalFiat += item.fiatTotal;
      return res;
    }, {
      principalTotalFiat: 0,
      rewardTotalFiat: 0,
      liquidityTotalFiat: 0
    });
  }, [myLiquidity]);
  const onexBalancesRaw = React.useMemo(() => holdings.balances.filter((x) => x.asset.chain.name === hub.OnomyHubConstants.ONEX_CHAIN_ID), [holdings.balances]);
  const mainBalancesRaw = React.useMemo(() => holdings.balances.filter((x) => x.asset.chain.name !== hub.OnomyHubConstants.ONEX_CHAIN_ID), [holdings.balances]);
  const [onexBalances, onexBalancesStatus] = reactHub.useDetailedBalances(onexBalancesRaw);
  const [mainBalances, mainBalancesStatus] = reactHub.useDetailedBalances(mainBalancesRaw);
  const onexBalancesTotalFiat = reactHub.useFiatTotal(onexBalances);
  const mainBalancesTotalFiat = reactHub.useFiatTotal(mainBalances);
  const mainGrandTotalFiat = holdings.stakedTotalFiat + holdings.rewardTotalFiat + mainBalancesTotalFiat;
  const onexGrandTotalFiat = onexBalancesTotalFiat + liquidityTotalFiat;
  const grandTotalFiat = mainGrandTotalFiat + onexGrandTotalFiat;
  const [nomUsdtDay, nomUsdtDayStatus] = useOnexDayTradeFrame({
    baseAsset: onexNomAsset,
    quoteAsset: onexUsdtAsset
  });
  const [nomEthDay, nomUsdtEthStatus] = useOnexDayTradeFrame({
    baseAsset: onexNomAsset,
    quoteAsset: onexEthAsset
  });
  const [onexNomBurned, onexNomBurnedStatus] = useOnexBurned();
  const loadingStatus = reactUtils.useAsyncStatuses([
    appLoadingStatus,
    holdings.loadingStatus,
    nomUsdtDayStatus,
    nomUsdtEthStatus,
    myLiquidityStatus,
    delegationsStatus,
    onexBalancesStatus,
    mainBalancesStatus,
    onexNomBurnedStatus
  ]);
  return {
    holdings,
    myLiquidity,
    delegations,
    nomUsdtDay,
    nomEthDay,
    onexNomBurned,
    onexBalances,
    mainBalances,
    oneNomCoin,
    principalTotalFiat,
    rewardTotalFiat,
    liquidityTotalFiat,
    onexBalancesTotalFiat,
    mainBalancesTotalFiat,
    mainGrandTotalFiat,
    onexGrandTotalFiat,
    grandTotalFiat,
    loadingStatus
  };
}
const OnomyDashboardContext = React.createContext(null);
function useOnomyDashboard() {
  const state = React.useContext(OnomyDashboardContext);
  if (!state)
    throw new Error("OnomyDashboardContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$8() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading dashboard"
  });
}
function DEFAULT_ERROR_RENDER_FN$8(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function OnomyDashboardProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$8,
  renderError = DEFAULT_ERROR_RENDER_FN$8
}) {
  const state = useOnomyDashboardState();
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(OnomyDashboardContext.Provider, {
    value: state
  }, children);
}

const PERIOD_MAP = {
  "1": "Minute",
  "15": "QuarterHour",
  "60": "Hour",
  "480": "QuarterDay",
  "1D": "Day",
  "1W": "Week"
};
const ONEX_SYMBOL_INFO = {
  name: "bNOM",
  full_name: "bNOM",
  description: "bNOM",
  exchange: "ONEX",
  listed_exchange: "ONEX",
  format: "price",
  has_intraday: true,
  has_no_volume: true,
  has_seconds: false,
  has_ticks: false,
  has_daily: true,
  has_empty_bars: true,
  data_status: "streaming",
  original_currency_code: "ETH",
  currency_code: "ETH",
  supported_resolutions: Object.keys(PERIOD_MAP),
  intraday_multipliers: Object.keys(PERIOD_MAP),
  ticker: "bNOM",
  timezone: "Etc/UTC",
  type: "crypto",
  session: "24x7",
  minmov: 1,
  pricescale: 1e5
};
class OnexTradingViewData {
  constructor(onexHistoryFrames, baseDenom, quoteDenom) {
    this.onexHistoryFrames = onexHistoryFrames;
    this.barSubs = {};
    this.lastTick = {};
    this.baseDenom = baseDenom;
    this.quoteDenom = quoteDenom;
    this.onBlockMined = this.onBlockMined.bind(this);
    this.timer = setInterval(this.onBlockMined.bind(this), 5e3);
  }
  async onBlockMined() {
    const subs = Object.values(this.barSubs);
    for (const { symbolInfo, resolution, onTick } of subs) {
      this.getBars(symbolInfo, resolution, { countBack: 1, from: 0, to: 0, firstDataRequest: false }, (res) => {
        const last = res.slice().pop();
        if (last && (!this.lastTick[resolution] || this.lastTick[resolution] < last.time)) {
          onTick(last);
        }
      }, (err) => console.error(err));
    }
  }
  destroy() {
    clearInterval(this.timer);
  }
  onReady(cb) {
    setTimeout(() => cb({
      exchanges: [
        {
          value: ONEX_SYMBOL_INFO.exchange,
          name: ONEX_SYMBOL_INFO.exchange,
          desc: ONEX_SYMBOL_INFO.exchange
        }
      ],
      symbols_types: [
        {
          value: ONEX_SYMBOL_INFO.type,
          name: ONEX_SYMBOL_INFO.type
        }
      ],
      supported_resolutions: ONEX_SYMBOL_INFO.supported_resolutions,
      currency_codes: ["ETH"],
      units: {},
      supports_marks: false,
      supports_timescale_marks: false,
      supports_time: true
    }), 0);
  }
  async searchSymbols(userInput, exchange, symbolType, onResult) {
    onResult([
      {
        symbol: ONEX_SYMBOL_INFO.ticker,
        full_name: ONEX_SYMBOL_INFO.name,
        description: ONEX_SYMBOL_INFO.description,
        exchange: ONEX_SYMBOL_INFO.exchange,
        ticker: ONEX_SYMBOL_INFO.ticker,
        type: ONEX_SYMBOL_INFO.type
      }
    ]);
  }
  resolveSymbol(symbolName, onSuccess) {
    setTimeout(() => onSuccess(ONEX_SYMBOL_INFO), 0);
  }
  async getBars(symbolInfo, resolution, periodParams, onResult, onError) {
    var _a;
    try {
      const frames = await this.onexHistoryFrames.get({
        frameType: PERIOD_MAP[resolution] || "Hour",
        skipped: 0,
        baseDenom: this.baseDenom,
        quoteDenom: this.quoteDenom,
        from: periodParams.from,
        to: (_a = periodParams.to) != null ? _a : void 0,
        first: Math.min(1e3, periodParams.countBack)
      });
      const bars = frames.slice().reverse().map((frame) => {
        const time = parseInt(frame.startTime, 10) * 1e3;
        if (!this.lastTick[resolution] || this.lastTick[resolution] <= time) {
          this.lastTick[resolution] = time;
        } else {
        }
        return {
          time,
          open: frame.startPrice,
          high: frame.maxPrice,
          low: frame.minPrice,
          close: frame.endPrice,
          volume: frame.volume
        };
      });
      if (bars.length > 0) {
        onResult(bars, { noData: false });
      } else {
        onResult([], { noData: true });
      }
    } catch (e) {
      console.error(e);
      onError(`${e}`);
    }
  }
  subscribeBars(symbolInfo, resolution, onTick, listenerGuid, onResetCacheNeededCallback) {
    this.barSubs[listenerGuid] = {
      symbolInfo,
      resolution,
      onTick,
      listenerGuid,
      onResetCacheNeededCallback
    };
  }
  unsubscribeBars(listenerGuid) {
    delete this.barSubs[listenerGuid];
  }
}
function useOnexTradingViewData(baseDenom, quoteDenom) {
  const { onexHistoryFrames } = useOnomyApp();
  try {
    return React.useMemo(() => {
      if (!onexHistoryFrames)
        return null;
      return new OnexTradingViewData(onexHistoryFrames, baseDenom, quoteDenom);
    }, [onexHistoryFrames, baseDenom, quoteDenom]);
  } catch (e) {
    console.warn("useOnexTradeData error", e);
    return null;
  }
}

const BN_ZERO = new BigNumber__default["default"](0);
const BN_ONE = new BigNumber__default["default"](1);
const DEFAULT_STATE = {
  rate: ["0", "0"],
  price: "0",
  inversePrice: "0",
  amountLastChanged: "base",
  amountBaseFixed: "0",
  amountQuoteFixed: "0"
};
const MAX_RATE_DENOMINATOR = "18446744073709551614";
function calculateRate({
  price,
  baseAsset,
  quoteAsset
}) {
  let bnPrice = new BigNumber__default["default"](price);
  if (bnPrice.isNaN())
    bnPrice = BN_ZERO;
  const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
  let bnInversePrice = BN_ONE.dividedBy(bnPrice).shiftedBy(-1 * expShift);
  if (!bnInversePrice.isFinite())
    bnInversePrice = BN_ZERO;
  bnPrice = bnPrice.shiftedBy(expShift);
  const rate = bnPrice.eq(0) ? ["0", "0"] : bnPrice.toFraction(MAX_RATE_DENOMINATOR).map((x) => x.toFixed(0));
  while (BigNumber__default["default"](rate[0]).gte(MAX_RATE_DENOMINATOR)) {
    rate[0] = rate[0].slice(0, rate[0].length - 1);
    rate[1] = rate[1].slice(0, rate[1].length - 1);
  }
  return {
    price,
    bnPrice,
    bnInversePrice,
    rate,
    inversePrice: bnInversePrice.toString(10)
  };
}
function calculatePrice({
  baseAmountFixed,
  quoteAmountFixed,
  baseAsset,
  quoteAsset
}) {
  var _a, _b;
  const bnQuote = new BigNumber__default["default"](quoteAmountFixed).shiftedBy(-1 * ((_a = quoteAsset.meta.decimalPlaces) != null ? _a : 18));
  const bnBase = new BigNumber__default["default"](baseAmountFixed).shiftedBy(-1 * ((_b = baseAsset.meta.decimalPlaces) != null ? _b : 18));
  const price = bnQuote.dividedBy(bnBase).toString(10);
  return calculateRate({ price, baseAsset, quoteAsset });
}
function usePriceInputs({
  baseAsset,
  quoteAsset,
  updateOtherAsset = true
}) {
  const [{ rate, price, inversePrice, amountBaseFixed, amountQuoteFixed }, setState] = React.useState(DEFAULT_STATE);
  const priceFixed = useFixedAmountFromDisplay(price, quoteAsset);
  const [pricePerBase, pricePerBaseStatus] = reactHub.useDetailedBalance(quoteAsset, priceFixed);
  const inversePriceFixed = useFixedAmountFromDisplay(inversePrice, baseAsset);
  const [inversePricePerBase, inversePricePerBaseStatus] = reactHub.useDetailedBalance(baseAsset, inversePriceFixed);
  const setPrice = React.useCallback((newPrice) => {
    setState((current) => {
      const {
        bnPrice: calculatedBnPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate
      } = calculateRate({
        price: newPrice,
        baseAsset,
        quoteAsset
      });
      if (current.amountLastChanged === "base") {
        const bnAmount2 = new BigNumber__default["default"](current.amountBaseFixed);
        const calculatedQuote2 = bnAmount2.multipliedBy(calculatedBnPrice);
        return {
          ...current,
          amountQuoteFixed: calculatedQuote2.isNaN() || !calculatedQuote2.isFinite() ? "0" : calculatedQuote2.toFixed(0),
          rate: calculatedRate,
          price: newPrice,
          inversePrice: calculatedInversePrice
        };
      }
      const bnAmount = new BigNumber__default["default"](current.amountQuoteFixed);
      const calculatedQuote = bnAmount.dividedBy(calculatedBnPrice);
      return {
        ...current,
        amountBaseFixed: calculatedQuote.isNaN() || !calculatedQuote.isFinite() ? "0" : calculatedQuote.toFixed(0),
        rate: calculatedRate,
        price: newPrice,
        inversePrice: calculatedInversePrice
      };
    });
  }, [baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]);
  const setAmountBaseFixed = React.useCallback((newAmountFixed) => {
    setState((current) => {
      if (updateOtherAsset) {
        const bnAmount = new BigNumber__default["default"](newAmountFixed);
        const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
        const bnPrice = new BigNumber__default["default"](current.price).shiftedBy(expShift);
        const calculatedQuote = bnAmount.multipliedBy(bnPrice);
        return {
          ...current,
          amountLastChanged: "base",
          amountBaseFixed: newAmountFixed,
          amountQuoteFixed: calculatedQuote.isNaN() || !calculatedQuote.isFinite() ? "0" : calculatedQuote.toFixed(0)
        };
      }
      const {
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate
      } = calculatePrice({
        baseAsset,
        quoteAsset,
        baseAmountFixed: newAmountFixed,
        quoteAmountFixed: current.amountQuoteFixed
      });
      return {
        ...current,
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate,
        amountLastChanged: "base",
        amountBaseFixed: newAmountFixed
      };
    });
  }, [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]);
  const setAmountQuoteFixed = React.useCallback((newAmountFixed) => {
    setState((current) => {
      if (updateOtherAsset) {
        const bnAmount = new BigNumber__default["default"](newAmountFixed);
        const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
        const bnPrice = new BigNumber__default["default"](current.price).shiftedBy(expShift);
        const calculatedBase = bnAmount.dividedBy(bnPrice);
        return {
          ...current,
          amountLastChanged: "quote",
          amountQuoteFixed: newAmountFixed,
          amountBaseFixed: calculatedBase.isNaN() || !calculatedBase.isFinite() ? "0" : calculatedBase.toFixed(0)
        };
      }
      const {
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate
      } = calculatePrice({
        baseAsset,
        quoteAsset,
        quoteAmountFixed: newAmountFixed,
        baseAmountFixed: current.amountBaseFixed
      });
      return {
        ...current,
        price: calculatedPrice,
        inversePrice: calculatedInversePrice,
        rate: calculatedRate,
        amountLastChanged: "quote",
        amountQuoteFixed: newAmountFixed
      };
    });
  }, [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]);
  const reset = React.useCallback(() => setState(DEFAULT_STATE), []);
  return {
    rate,
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    priceFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePriceFixed,
    inversePricePerBase,
    inversePricePerBaseStatus,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    reset
  };
}

function useCreateOrderForm({
  baseDenom = "anom",
  quoteDenom = "usdt",
  orderDirection = "buy",
  orderType: orderTypeProp = "limit"
}) {
  var _a;
  const onex = useOnexClient();
  const chain = (_a = onex == null ? void 0 : onex.getBlockchain()) != null ? _a : null;
  const [direction, setDirection] = React.useState(orderDirection);
  const [baseAsset, baseAssetStatus] = useAsset(baseDenom, chain);
  const [quoteAsset, quoteAssetStatus] = useAsset(quoteDenom, chain);
  const assetStatus = reactUtils.useAsyncStatuses([baseAssetStatus, quoteAssetStatus]);
  const {
    rate,
    price,
    amountBaseFixed,
    amountQuoteFixed,
    pricePerBase,
    pricePerBaseStatus,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    reset: resetPriceInputs
  } = usePriceInputs({ baseAsset, quoteAsset });
  const [stopPrice, setStopPrice] = React.useState("0");
  const stopPriceFixed = useFixedAmountFromDisplay(stopPrice, quoteAsset);
  const [stopPricePerBase, stopPricePerBaseStatus] = reactHub.useDetailedBalance(quoteAsset, stopPriceFixed);
  const [currentPricePerBase, currentPricePerBaseStatus] = useOnexDetailedPoolPrice({
    baseAsset,
    quoteAsset
  });
  const [baseBalance, baseBalanceStatus] = reactHub.useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = reactHub.useAssetBalance(quoteAsset);
  const balanceStatus = reactUtils.useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const [orderType, setOrderType] = React.useState(orderTypeProp);
  const [txResult, setTxResult] = React.useState(null);
  const [attemptTx, txStatus, resetTxStatus] = reactUtils.useAsyncProcess();
  const [amountQuote, amountQuoteStatus] = reactHub.useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = reactHub.useDetailedBalance(baseAsset, amountBaseFixed);
  const amountStatus = reactUtils.useAsyncStatuses([amountQuoteStatus, amountBaseStatus]);
  const [bookSell, bookSellStatus] = useOnexOrderBook({ baseAsset, quoteAsset, direction: "sell" });
  const [bookBuy, bookBuyStatus] = useOnexOrderBook({ baseAsset, quoteAsset, direction: "buy" });
  const bookStatus = reactUtils.useAsyncStatuses([bookSellStatus, bookBuyStatus]);
  const bidAsset = direction === "buy" ? quoteAsset : baseAsset;
  const askAsset = direction === "buy" ? baseAsset : quoteAsset;
  const bidBalance = direction === "buy" ? quoteBalance : baseBalance;
  const askBalance = direction === "buy" ? baseBalance : quoteBalance;
  const bidAmountFixed = direction === "buy" ? amountQuoteFixed : amountBaseFixed;
  const askAmountFixed = direction === "buy" ? amountBaseFixed : amountQuoteFixed;
  const bidAmount = direction === "buy" ? amountQuote : amountBase;
  const askAmount = direction === "buy" ? amountBase : amountQuote;
  const setBidAmountFixed = direction === "buy" ? setAmountQuoteFixed : setAmountBaseFixed;
  const setAskAmountFixed = direction === "buy" ? setAmountBaseFixed : setAmountQuoteFixed;
  const inputParams = React.useMemo(() => {
    if (direction === "buy") {
      return {
        denomAsk: baseDenom,
        denomBid: quoteDenom,
        orderType,
        amount: amountQuoteFixed,
        rate: [rate[1], rate[0]]
      };
    }
    return {
      denomAsk: quoteDenom,
      denomBid: baseDenom,
      orderType,
      amount: amountBaseFixed,
      rate: [rate[0], rate[1]]
    };
  }, [direction, baseDenom, quoteDenom, orderType, amountQuoteFixed, amountBaseFixed, rate]);
  const validationErrors = React.useMemo(() => {
    const errors = {};
    const bnPrice = new BigNumber__default["default"](price);
    const bnAmountQuote = new BigNumber__default["default"](amountQuoteFixed);
    const bnAmountBase = new BigNumber__default["default"](amountBaseFixed);
    if (bnPrice.isNaN() || bnPrice.lte(0)) {
      errors.price = "Must be a number > 0";
    }
    if (bnAmountQuote.isNaN() || bnAmountQuote.lte(0)) {
      errors.amountQuote = "Must be a number > 0";
    }
    if (bnAmountBase.isNaN() || bnAmountBase.lte(0)) {
      errors.amountBase = "Must be a number > 0";
    }
    if (direction === "buy" && bnAmountQuote.gt(quoteBalance.amount.toString())) {
      errors.amountQuote = "Exceeds available balance";
    }
    if (direction === "sell" && bnAmountBase.gt(baseBalance.amount.toString())) {
      errors.amountBase = "Exceeds available balance";
    }
    if (direction === "buy") {
      if (errors.amountBase)
        errors.amountAsk = errors.amountBase;
      if (errors.amountQuote)
        errors.amountBid = errors.amountQuote;
    } else {
      if (errors.amountBase)
        errors.amountBid = errors.amountBase;
      if (errors.amountQuote)
        errors.amountAsk = errors.amountQuote;
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    direction,
    price,
    amountQuoteFixed,
    amountBaseFixed,
    quoteBalance.amount,
    baseBalance.amount
  ]);
  const loadingStatus = reactUtils.useAsyncStatuses([assetStatus, balanceStatus, currentPricePerBaseStatus]);
  const canSubmit = !validationErrors && loadingStatus.finished;
  const submitTx = React.useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a2;
      return setTxResult((_a2 = await (onex == null ? void 0 : onex.createOrder(inputParams))) != null ? _a2 : null);
    });
  }, [canSubmit, inputParams, resetTxStatus, attemptTx, onex]);
  const reset = React.useCallback(() => {
    resetPriceInputs();
    setOrderType("limit");
    resetTxStatus();
    setTxResult(null);
  }, [resetPriceInputs, resetTxStatus]);
  React.useEffect(reset, [baseDenom, quoteDenom, orderDirection, orderType, reset]);
  return {
    txStatus,
    txResult,
    assetStatus,
    balanceStatus,
    loadingStatus,
    amountStatus,
    pricePerBaseStatus,
    stopPricePerBaseStatus,
    currentPricePerBaseStatus,
    bookBuyStatus,
    bookSellStatus,
    bookStatus,
    validationErrors,
    canSubmit,
    baseAsset,
    baseBalance,
    quoteAsset,
    quoteBalance,
    amountQuoteFixed,
    amountBaseFixed,
    baseDenom,
    quoteDenom,
    direction,
    price,
    stopPrice,
    orderType,
    pricePerBase,
    stopPricePerBase,
    currentPricePerBase,
    amountQuote,
    amountBase,
    bookSell,
    bookBuy,
    bidAsset,
    askAsset,
    bidBalance,
    askBalance,
    bidAmountFixed,
    askAmountFixed,
    bidAmount,
    askAmount,
    setBidAmountFixed,
    setAskAmountFixed,
    setDirection,
    setPrice,
    setOrderType,
    setAmountQuoteFixed,
    setAmountBaseFixed,
    submitTx,
    reset
  };
}

function useMarketOrderForm({
  baseDenom = "anom",
  quoteDenom = "usdt",
  orderDirection = "buy"
}) {
  var _a;
  const onex = useOnexClient();
  const chain = (_a = onex == null ? void 0 : onex.getBlockchain()) != null ? _a : null;
  const [direction, setDirection] = React.useState(orderDirection);
  const [baseAsset, baseAssetStatus] = useAsset(baseDenom, chain);
  const [quoteAsset, quoteAssetStatus] = useAsset(quoteDenom, chain);
  const assetStatus = reactUtils.useAsyncStatuses([baseAssetStatus, quoteAssetStatus]);
  const [baseBalance, baseBalanceStatus] = reactHub.useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = reactHub.useAssetBalance(quoteAsset);
  const balanceStatus = reactUtils.useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const [txResult, setTxResult] = React.useState(null);
  const [attemptTx, txStatus, resetTxStatus] = reactUtils.useAsyncProcess();
  const [hasAttemptedSubmit, setHasAttemptedSubmit] = React.useState(false);
  const [bidAmountFixed, setBidAmountFixed] = React.useState("0");
  const [slippagePercentage, setSlippagePercentage] = React.useState("3");
  const [estimatedAskAmount, estimateStatus] = useOnexMarketEstimate({
    denomAsk: direction === "buy" ? baseDenom : quoteDenom,
    denomBid: direction === "buy" ? quoteDenom : baseDenom,
    amountBid: bidAmountFixed
  });
  const bidAsset = direction === "buy" ? quoteAsset : baseAsset;
  const askAsset = direction === "buy" ? baseAsset : quoteAsset;
  const bidBalance = direction === "buy" ? quoteBalance : baseBalance;
  const askBalance = direction === "buy" ? baseBalance : quoteBalance;
  const [bidAmount, bidAmountStatus] = reactHub.useDetailedBalance(bidAsset, bidAmountFixed);
  const [askAmount, askAmountStatus] = reactHub.useDetailedBalance(askAsset, estimatedAskAmount);
  const amountStatus = reactUtils.useAsyncStatuses([estimateStatus, bidAmountStatus, askAmountStatus]);
  const baseAmount = direction === "buy" ? askAmount : bidAmount;
  const quoteAmount = direction === "buy" ? bidAmount : askAmount;
  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = reactHub.useDetailedBalance(quoteAsset, currentPriceFixed);
  const estimatedPricePerBaseFixed = React.useMemo(() => {
    const bnEstAskFixed = new BigNumber__default["default"](estimatedAskAmount);
    const bnBidAmountFixed = new BigNumber__default["default"](bidAmountFixed);
    if (bnEstAskFixed.isNaN() || bnEstAskFixed.eq(0) || bnBidAmountFixed.isNaN() || bnBidAmountFixed.eq(0)) {
      return "0";
    }
    if (direction === "buy") {
      const expShift2 = bidAsset.meta.decimalPlaces - askAsset.meta.decimalPlaces;
      const bnPrice2 = bnBidAmountFixed.dividedBy(bnEstAskFixed).shiftedBy(expShift2).shiftedBy(bidAsset.meta.decimalPlaces);
      return bnPrice2.toFixed(0);
    }
    const expShift = askAsset.meta.decimalPlaces - bidAsset.meta.decimalPlaces;
    const bnPrice = bnEstAskFixed.dividedBy(bnBidAmountFixed).shiftedBy(expShift).shiftedBy(askAsset.meta.decimalPlaces);
    return bnPrice.toFixed(0);
  }, [
    estimatedAskAmount,
    bidAmountFixed,
    bidAsset.meta.decimalPlaces,
    askAsset.meta.decimalPlaces,
    direction
  ]);
  const [estimatedPricePerBase, estimatedPricePerBaseStatus] = reactHub.useDetailedBalance(quoteAsset, estimatedPricePerBaseFixed);
  const inputParams = React.useMemo(() => {
    const slippage = new BigNumber__default["default"](slippagePercentage).times(100).toFixed(0);
    if (direction === "buy") {
      return {
        denomAsk: baseDenom,
        denomBid: quoteDenom,
        amountBid: bidAmountFixed,
        amountAsk: estimatedAskAmount,
        slippage
      };
    }
    return {
      denomAsk: quoteDenom,
      denomBid: baseDenom,
      amountBid: bidAmountFixed,
      amountAsk: estimatedAskAmount,
      slippage
    };
  }, [slippagePercentage, direction, baseDenom, quoteDenom, bidAmountFixed, estimatedAskAmount]);
  const validationErrors = React.useMemo(() => {
    const errors = {};
    const bnAmountBid = new BigNumber__default["default"](inputParams.amountBid);
    const bnAmountAsk = new BigNumber__default["default"](inputParams.amountAsk);
    const bnSlippage = new BigNumber__default["default"](inputParams.slippage);
    if (bnAmountBid.isNaN() || !bnAmountBid.isFinite() || bnAmountBid.lte(0)) {
      errors.amountBid = "Must be a number > 0";
    }
    if (bnAmountAsk.isNaN() || !bnAmountAsk.isFinite() || bnAmountAsk.lte(0)) {
      errors.amountAsk = "Must be a number > 0";
    }
    if (bnSlippage.isNaN() || !bnSlippage.isFinite() || bnSlippage.lte(0)) {
      errors.slippage = "Must be a number > 0";
    }
    if (bnSlippage.gte(1e4)) {
      errors.slippage = "Must be a number < 100";
    }
    if (estimateStatus.error) {
      errors.liquidity = "Insufficient liquidity in pool";
    }
    if (bnAmountBid.gt(bidBalance.amount.toString())) {
      errors.amountBid = "Exceeds available balance";
    }
    if (bnAmountAsk.gt(askBalance.amount.toString())) {
      errors.amountAsk = "Exceeds available balance";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [inputParams, estimateStatus.error, bidBalance.amount, askBalance.amount]);
  const loadingStatus = reactUtils.useAsyncStatuses([assetStatus, balanceStatus]);
  const canSubmit = !validationErrors && loadingStatus.finished && estimateStatus.finished;
  const submitTx = React.useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    setHasAttemptedSubmit(true);
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a2;
      return setTxResult((_a2 = await (onex == null ? void 0 : onex.marketOrder(inputParams))) != null ? _a2 : null);
    });
  }, [canSubmit, inputParams, resetTxStatus, attemptTx, onex]);
  const reset = React.useCallback(() => {
    setHasAttemptedSubmit(false);
    resetTxStatus();
    setBidAmountFixed("0");
    setTxResult(null);
  }, [resetTxStatus]);
  React.useEffect(reset, [baseDenom, quoteDenom, direction, reset]);
  React.useEffect(() => {
    setDirection(orderDirection);
  }, [orderDirection]);
  return {
    txStatus,
    txResult,
    assetStatus,
    balanceStatus,
    loadingStatus,
    estimateStatus,
    bidAmountStatus,
    askAmountStatus,
    amountStatus,
    currentPriceStatus,
    currentPricePerBaseStatus,
    estimatedPricePerAskStatus: estimatedPricePerBaseStatus,
    validationErrors,
    canSubmit,
    hasAttemptedSubmit,
    baseAsset,
    baseBalance,
    quoteAsset,
    quoteBalance,
    bidAsset,
    askAsset,
    bidAmount,
    askAmount,
    bidBalance,
    askBalance,
    baseAmount,
    quoteAmount,
    currentPricePerBase,
    estimatedPricePerBase,
    baseDenom,
    quoteDenom,
    direction,
    bidAmountFixed,
    slippagePercentage,
    setDirection,
    setBidAmountFixed,
    setSlippagePercentage,
    submitTx,
    reset
  };
}

const ONEX_ORDER_BOOKS_QUERY = client$1.gql`
  query orderBooks($base: String!, $quote: String!) {
    orderBooks(where: { base: $base, quote: $quote }) {
      id
      total
      bins
    }
  }
`;
const ONEX_ORDER_BOOK_INCREMENTS_QUERY = client$1.gql`
  query bookIncrements($book: String!, $bin: BigDecimal!) {
    bookIncrements(where: { book: $book, bin: $bin }, orderBy: place, orderDirection: desc) {
      id
      place
      orders
      amount
    }
  }
`;
function useOrderBookBins({
  apolloClient,
  baseAsset,
  quoteAsset
}) {
  const cb = React.useCallback(async () => {
    var _a, _b;
    const resp = await apolloClient.query({
      query: ONEX_ORDER_BOOKS_QUERY,
      variables: {
        base: baseAsset.id,
        quote: quoteAsset.id
      }
    });
    const {
      data: { orderBooks: books }
    } = resp;
    const normalized = books.map((book) => {
      return {
        ...book,
        bins: book.bins.map((binId) => binId.split("-").pop())
      };
    });
    const bins = ((_b = (_a = normalized[0]) == null ? void 0 : _a.bins) != null ? _b : []).sort((a, b) => {
      return parseFloat(a) - parseFloat(b);
    });
    return bins;
  }, [apolloClient, baseAsset.id, quoteAsset.id]);
  return reactUtils.useAsyncPoll(cb, []);
}
function useOrderBookIncrements({
  apolloClient,
  baseAsset,
  quoteAsset,
  bin,
  direction
}) {
  const { hub } = reactHub.useOnomyHub();
  const cb = React.useCallback(async () => {
    if (!bin || !hub)
      return [];
    const book = `${baseAsset.id}-${quoteAsset.id}-${direction}`;
    const resp = await apolloClient.query({
      query: ONEX_ORDER_BOOK_INCREMENTS_QUERY,
      variables: {
        book,
        bin
      }
    });
    let {
      data: { bookIncrements = [] }
    } = resp;
    let totalBase = new BigNumber__default["default"](0);
    try {
      if (direction === "sell")
        bookIncrements = bookIncrements.slice().reverse();
    } catch (e) {
      console.error(e);
    }
    const increments = bookIncrements.map((raw) => {
      const bnPlace = new BigNumber__default["default"](raw.place);
      const bnPricePer = bnPlace.shiftedBy(quoteAsset.meta.decimalPlaces);
      const bnQuoteAmount = bnPlace.times(raw.amount);
      totalBase = totalBase.plus(raw.amount);
      return {
        uid: raw.id,
        place: raw.place,
        amountRaw: raw.amount,
        orders: raw.orders,
        amount: {
          amount: BigInt(raw.amount),
          asset: baseAsset
        },
        totalAmount: {
          amount: BigInt(totalBase.toFixed(0)),
          asset: baseAsset
        },
        quoteAmount: {
          amount: BigInt(bnQuoteAmount.toFixed(0)),
          asset: quoteAsset
        },
        pricePer: {
          amount: BigInt(bnPricePer.toFixed(0)),
          asset: quoteAsset
        }
      };
    });
    if (direction === "sell")
      increments.reverse();
    const detailedIncrements = await Promise.all(increments.map(async (inc) => {
      const [amount, totalAmount, quoteAmount, pricePer] = await hub.balances.includeDetailsMultiple([
        inc.amount,
        inc.totalAmount,
        inc.quoteAmount,
        inc.pricePer
      ]);
      return {
        ...inc,
        percentage: new BigNumber__default["default"](totalAmount.amount.toString()).dividedBy(totalBase).times(100).toNumber(),
        amount,
        totalAmount,
        quoteAmount,
        pricePer
      };
    }));
    return detailedIncrements;
  }, [hub, apolloClient, baseAsset.id, quoteAsset.id, bin, direction]);
  return reactUtils.useAsyncPoll(cb, []);
}
function useIndexedOrderBookForm({
  baseAsset,
  quoteAsset
}) {
  var _a;
  const { ecosystem } = useOnomyApp();
  const apolloClient = React.useMemo(() => {
    return new client$1.ApolloClient({
      uri: ecosystem.onexGraphqlEndpoint,
      cache: new client$1.InMemoryCache()
    });
  }, [ecosystem.onexGraphqlEndpoint]);
  const [orderBins, orderBinsStatus] = useOrderBookBins({
    apolloClient,
    baseAsset,
    quoteAsset
  });
  const binOptions = React.useMemo(() => orderBins.map((bin) => {
    const val = parseFloat(bin);
    return {
      value: bin,
      label: val > 1 ? val.toLocaleString() : val.toString()
    };
  }), [orderBins]);
  const [selectedBin, setSelectedBin] = React.useState(void 0);
  React.useMemo(() => {
    if (binOptions[0] && typeof selectedBin === "undefined") {
      setSelectedBin(binOptions[0].value);
    }
  }, [(_a = binOptions[0]) == null ? void 0 : _a.value]);
  const [bookBuy, bookBuyStatus] = useOrderBookIncrements({
    apolloClient,
    baseAsset,
    quoteAsset,
    bin: selectedBin != null ? selectedBin : "",
    direction: "buy"
  });
  const [bookSell, bookSellStatus] = useOrderBookIncrements({
    apolloClient,
    baseAsset,
    quoteAsset,
    bin: selectedBin != null ? selectedBin : "",
    direction: "sell"
  });
  const loadingStatus = reactUtils.useAsyncStatuses([orderBinsStatus, bookBuyStatus, bookSellStatus]);
  const chartData = React.useMemo(() => {
    const bids = bookBuy.map((item) => {
      return {
        side: "buy",
        price: parseFloat(item.pricePer.amountFormatted),
        amount: parseFloat(item.amount.amountFormatted)
      };
    });
    const asks = bookSell.map((item) => {
      return {
        side: "sell",
        price: parseFloat(item.pricePer.amountFormatted),
        amount: parseFloat(item.amount.amountFormatted)
      };
    });
    return {
      market: `${baseAsset.meta.symbol}/${quoteAsset.meta.symbol}`,
      bids,
      asks
    };
  }, [bookBuy, bookSell, baseAsset.meta.symbol, quoteAsset.meta.symbol]);
  return {
    loadingStatus,
    orderBinsStatus,
    bookBuyStatus,
    bookSellStatus,
    binOptions,
    selectedBin,
    bookBuy,
    bookSell,
    chartData,
    setSelectedBin
  };
}

function useExchangePairState({
  quoteAsset,
  baseAsset
}) {
  const createOrderForm = useCreateOrderForm({
    quoteDenom: quoteAsset.id,
    baseDenom: baseAsset.id
  });
  const marketOrderForm = useMarketOrderForm({
    quoteDenom: quoteAsset.id,
    baseDenom: baseAsset.id,
    orderDirection: createOrderForm.direction
  });
  const [dayTradeData, dayTradeDataStatus] = useOnexDayTradeFrame({ baseAsset, quoteAsset });
  const [history, historyStatus] = useOnexOrderHistory({ baseAsset, quoteAsset });
  const [pool, poolStatus] = useOnexPoolDetail({ baseAsset, quoteAsset });
  const [orders, ordersStatus] = useOnexMyActiveOrders({ quoteAsset, baseAsset });
  const {
    binOptions: bookBinOptions,
    bookBuy,
    bookSell,
    selectedBin: bookSelectedBin,
    setSelectedBin: setBookSelectedBin
  } = useIndexedOrderBookForm({
    baseAsset: createOrderForm.baseAsset,
    quoteAsset: createOrderForm.quoteAsset
  });
  const loadingStatus = reactUtils.useAsyncStatuses([
    createOrderForm.loadingStatus,
    dayTradeDataStatus,
    dayTradeData.loadingStatus,
    historyStatus,
    poolStatus,
    ordersStatus
  ]);
  return {
    loadingStatus,
    dayTradeDataStatus,
    quoteAsset,
    baseAsset,
    currentPricePerBase: createOrderForm.currentPricePerBase,
    lastPricePerBase: dayTradeData.lastPricePerBase,
    dayTradeData,
    createOrderForm,
    marketOrderForm,
    history,
    pool,
    orders,
    bookBuy,
    bookSell,
    bookBinOptions,
    bookSelectedBin,
    orderDirection: createOrderForm.direction,
    setOrderDirection: createOrderForm.setDirection,
    setBookSelectedBin
  };
}
const ExchangePairContext = React.createContext(null);
function useExchangePair() {
  const state = React.useContext(ExchangePairContext);
  if (!state)
    throw new Error("ExchangePairContext missing");
  return state;
}
const DEFAULT_LOADING_RENDER_FN$7 = () => /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
  text: "Loading exchange pair"
});
const DEFAULT_ERROR_RENDER_FN$7 = (error) => /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
  error
});
function ExchangePairProviderInner({
  children,
  quoteAsset,
  baseAsset,
  renderLoading = DEFAULT_LOADING_RENDER_FN$7,
  renderError = DEFAULT_ERROR_RENDER_FN$7
}) {
  const state = useExchangePairState({ baseAsset, quoteAsset });
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(ExchangePairContext.Provider, {
    value: state
  }, children);
}
function ExchangePairProvider({
  children,
  getBaseSymbol,
  getQuoteSymbol,
  renderLoading = DEFAULT_LOADING_RENDER_FN$7,
  renderError = DEFAULT_ERROR_RENDER_FN$7
}) {
  const baseSymbol = getBaseSymbol();
  const quoteSymbol = getQuoteSymbol();
  const [allAssets, assetStatus] = reactHub.useSupportedAssetList();
  const onexAssets = React.useMemo(() => allAssets.filter((x) => x.chain.name === hub.OnomyHubConstants.ONEX_CHAIN_ID), [allAssets]);
  const baseAsset = React.useMemo(() => onexAssets.find((x) => x.meta.symbol.toLowerCase() === baseSymbol), [onexAssets, baseSymbol]);
  const quoteAsset = React.useMemo(() => onexAssets.find((x) => x.meta.symbol.toLowerCase() === quoteSymbol), [onexAssets, quoteSymbol]);
  if (assetStatus.pending)
    return renderLoading();
  if (assetStatus.error)
    return renderError(assetStatus.error);
  if (!baseAsset || !quoteAsset)
    return /* @__PURE__ */ React__default["default"].createElement("h2", null, "Pair not found");
  return /* @__PURE__ */ React__default["default"].createElement(ExchangePairProviderInner, {
    baseAsset,
    quoteAsset,
    renderError,
    renderLoading
  }, children);
}

const DEFAULT_AMOUNT_INPUTS_STATE = {
  amountBaseInputFixed: "0",
  amountQuoteInputFixed: "0",
  amountLastChanged: "base"
};
function useAddDropsForm({
  baseAsset,
  quoteAsset,
  pool
}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = reactUtils.useAsyncProcess();
  const [txResult, setTxResult] = React.useState(null);
  const [baseBalance, baseBalanceStatus] = reactHub.useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = reactHub.useAssetBalance(quoteAsset);
  const balanceStatus = reactUtils.useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const [currentPricePerBase, currentPricePerBaseStatus] = useOnexDetailedPoolPrice({
    baseAsset,
    quoteAsset
  });
  const [currentPricePerQuote, currentPricePerQuoteStatus] = useOnexDetailedPoolPrice({
    baseAsset: quoteAsset,
    quoteAsset: baseAsset
  });
  const pair = React.useMemo(() => [baseAsset.id, quoteAsset.id].sort().join(","), [baseAsset.id, quoteAsset.id]);
  const [{ amountBaseInputFixed, amountQuoteInputFixed, amountLastChanged }, setInputState] = React.useState(DEFAULT_AMOUNT_INPUTS_STATE);
  const setAmountBaseFixed = React.useCallback((newAmountFixed) => {
    setInputState((state) => ({
      ...state,
      amountBaseInputFixed: newAmountFixed,
      amountLastChanged: "base"
    }));
  }, []);
  const setAmountQuoteFixed = React.useCallback((newAmountFixed) => {
    setInputState((state) => ({
      ...state,
      amountQuoteInputFixed: newAmountFixed,
      amountLastChanged: "quote"
    }));
  }, []);
  const setAmountBasePercentOfBalance = React.useCallback((percentage) => {
    const bnBalance = new BigNumber__default["default"](baseBalance.amount.toString()).times(percentage / 100);
    setAmountBaseFixed(bnBalance.toFixed(0));
  }, [baseBalance.amount, setAmountBaseFixed]);
  const setAmountQuotePercentOfBalance = React.useCallback((percentage) => {
    const bnBalance = new BigNumber__default["default"](quoteBalance.amount.toString()).times(percentage / 100);
    setAmountQuoteFixed(bnBalance.toFixed(0));
  }, [quoteBalance.amount, setAmountQuoteFixed]);
  const [{ amountB: calculatedOtherAmount, drops }, dropsStatus] = reactUtils.useAsyncPoll(React.useCallback(async () => {
    var _a;
    return (_a = onex == null ? void 0 : onex.dropCoin({
      denomA: amountLastChanged === "base" ? baseAsset.id : quoteAsset.id,
      denomB: amountLastChanged === "base" ? quoteAsset.id : baseAsset.id,
      amountA: amountLastChanged === "base" ? amountBaseInputFixed : amountQuoteInputFixed
    })) != null ? _a : {
      amountB: "0",
      drops: "0"
    };
  }, [
    onex,
    amountBaseInputFixed,
    amountQuoteInputFixed,
    amountLastChanged,
    baseAsset.id,
    quoteAsset.id
  ]), {
    amountB: "0",
    drops: "0"
  });
  const dropsToPoolRatio = React.useMemo(() => {
    var _a;
    const bnPoolDrops = new BigNumber__default["default"]((_a = pool == null ? void 0 : pool.drops) != null ? _a : 0);
    const bnDrops = new BigNumber__default["default"](drops);
    const bnTotal = bnDrops.plus(bnPoolDrops);
    return bnDrops.dividedBy(bnTotal).toNumber();
  }, [pool == null ? void 0 : pool.drops, drops]);
  const amountBaseFixed = amountLastChanged === "base" ? amountBaseInputFixed : calculatedOtherAmount;
  const amountQuoteFixed = amountLastChanged === "quote" ? amountQuoteInputFixed : calculatedOtherAmount;
  const [amountQuote, amountQuoteStatus] = reactHub.useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = reactHub.useDetailedBalance(baseAsset, amountBaseFixed);
  const inputParams = React.useMemo(() => {
    return {
      pair,
      drops
    };
  }, [pair, drops]);
  const validationErrors = React.useMemo(() => {
    const errors = {};
    const bnDrops = new BigNumber__default["default"](drops);
    const bnAmountBase = new BigNumber__default["default"](amountBaseFixed);
    const bnAmountQuote = new BigNumber__default["default"](amountQuoteFixed);
    if (bnDrops.isNaN() || bnDrops.lte(0)) {
      errors.drops = "Drops must be a number > 0";
    }
    if (bnAmountBase.isNaN() || bnAmountBase.lte(0)) {
      errors.amountBaseFixed = "Must be a number > 0";
    }
    if (bnAmountQuote.isNaN() || bnAmountQuote.lte(0)) {
      errors.amountQuoteFixed = "Must be a number > 0";
    }
    if (bnAmountBase.gt(baseBalance.amount.toString())) {
      errors.amountBaseFixed = "Exceeds available balance";
    }
    if (bnAmountQuote.gt(quoteBalance.amount.toString())) {
      errors.amountQuoteFixed = "Exceeds available balance";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [drops, amountBaseFixed, amountQuoteFixed, baseBalance.amount, quoteBalance.amount]);
  const loadingStatus = reactUtils.useAsyncStatuses([balanceStatus]);
  const canSubmit = !validationErrors && !loadingStatus.pending && dropsStatus.finished;
  const submitTx = React.useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.createDrop(inputParams))) != null ? _a : null);
    });
  }, [onex, inputParams, canSubmit, resetTxStatus, attemptTx]);
  const reset = React.useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);
  React.useEffect(reset, [baseAsset.id, quoteAsset.id, reset]);
  return {
    txStatus,
    txResult,
    loadingStatus,
    baseBalanceStatus,
    quoteBalanceStatus,
    balanceStatus,
    currentPricePerBaseStatus,
    currentPricePerQuoteStatus,
    amountBaseStatus,
    amountQuoteStatus,
    dropsStatus,
    baseAsset,
    quoteAsset,
    drops,
    amountBaseFixed,
    amountQuoteFixed,
    currentPricePerBase,
    currentPricePerQuote,
    amountBase,
    amountQuote,
    baseBalance,
    quoteBalance,
    dropsToPoolRatio,
    validationErrors,
    canSubmit,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    setAmountBasePercentOfBalance,
    setAmountQuotePercentOfBalance,
    submitTx,
    reset
  };
}

function useOnexLiquidity({
  baseAsset,
  quoteAsset
}) {
  const unknownBaseBalance = React.useMemo(() => ({ ...hub.OnomyHubConstants.UNKNOWN_BALANCE, asset: baseAsset }), [baseAsset.id, baseAsset.chain.name]);
  const unknownQuoteBalance = React.useMemo(() => ({ ...hub.OnomyHubConstants.UNKNOWN_BALANCE, asset: quoteAsset }), [quoteAsset.id, baseAsset.chain.name]);
  const [pool, poolStatus] = useOnexPoolDetail({ baseAsset, quoteAsset });
  const [myDrops, myDropsStatus] = useOnexMyDropsForPool(pool);
  const basePrincipalAmounts = React.useMemo(() => myDrops.map((drop) => drop.principalBase), [myDrops]);
  const quotePrincipalAmounts = React.useMemo(() => myDrops.map((drop) => drop.principalQuote), [myDrops]);
  const baseRewardAmounts = React.useMemo(() => myDrops.map((drop) => drop.rewardBase), [myDrops]);
  const quoteRewardAmounts = React.useMemo(() => myDrops.map((drop) => drop.rewardQuote), [myDrops]);
  const [[basePrincipal = unknownBaseBalance], basePrincipalStatus] = reactHub.useBalanceTotals(basePrincipalAmounts);
  const [[quotePrincipal = unknownQuoteBalance], quotePrincipalStatus] = reactHub.useBalanceTotals(quotePrincipalAmounts);
  const [[baseReward = unknownBaseBalance], baseRewardStatus] = reactHub.useBalanceTotals(baseRewardAmounts);
  const [[quoteReward = unknownQuoteBalance], quoteRewardStatus] = reactHub.useBalanceTotals(quoteRewardAmounts);
  const [[baseTotal = unknownBaseBalance], baseTotalStatus] = reactHub.useBalanceTotals([
    basePrincipal,
    baseReward
  ]);
  const [[quoteTotal = unknownQuoteBalance], quoteTotalStatus] = reactHub.useBalanceTotals([
    quotePrincipal,
    quoteReward
  ]);
  const fiatTotal = baseTotal.fiatValue + quoteTotal.fiatValue;
  const totalsStatus = reactUtils.useAsyncStatuses([
    basePrincipalStatus,
    quotePrincipalStatus,
    baseRewardStatus,
    quoteRewardStatus,
    baseTotalStatus,
    quoteTotalStatus
  ]);
  const totalRewardRatio = React.useMemo(() => {
    const bnPrinciple = new BigNumber__default["default"](basePrincipal.amount.toString());
    const bnReward = new BigNumber__default["default"](baseReward.amount.toString());
    const bnTotal = bnReward.plus(bnPrinciple);
    return bnReward.dividedBy(bnTotal).toNumber();
  }, [basePrincipal.amount, baseReward.amount]);
  const loadingStatus = reactUtils.useAsyncStatuses([poolStatus, myDropsStatus, totalsStatus]);
  return {
    loadingStatus,
    poolStatus,
    myDropsStatus,
    basePrincipalStatus,
    quotePrincipalStatus,
    baseRewardStatus,
    quoteRewardStatus,
    totalsStatus,
    pool,
    myDrops,
    basePrincipal,
    quotePrincipal,
    baseReward,
    quoteReward,
    baseTotal,
    quoteTotal,
    fiatTotal,
    totalRewardRatio
  };
}

function useLiquidityPairState({
  quoteAsset,
  baseAsset
}) {
  const liquidity = useOnexLiquidity({ baseAsset, quoteAsset });
  return {
    ...liquidity,
    baseAsset,
    quoteAsset
  };
}
const LiquidityPairContext = React.createContext(null);
function useLiquidityPair() {
  const state = React.useContext(LiquidityPairContext);
  if (!state)
    throw new Error("LiquidityPairContext missing");
  return state;
}
const DEFAULT_LOADING_RENDER_FN$6 = () => /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
  text: "Loading liquidity pair"
});
const DEFAULT_ERROR_RENDER_FN$6 = (error) => /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
  error
});
function LiquidityPairProvider({
  quoteAsset,
  baseAsset,
  renderLoading = DEFAULT_LOADING_RENDER_FN$6,
  renderError = DEFAULT_ERROR_RENDER_FN$6,
  children
}) {
  const state = useLiquidityPairState({ baseAsset, quoteAsset });
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(LiquidityPairContext.Provider, {
    value: state
  }, children);
}

function useCancelOrderForm({ orderId = "" } = {}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = reactUtils.useAsyncProcess();
  const [txResult, setTxResult] = React.useState(null);
  const [uid, setUid] = React.useState(orderId);
  const doCancel = React.useCallback((id) => {
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.cancelOrder({ uid: id }))) != null ? _a : null);
    });
  }, [onex, resetTxStatus, attemptTx]);
  const submitTx = React.useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    return doCancel(uid);
  }, [doCancel, uid]);
  const reset = React.useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);
  React.useEffect(() => {
    setUid(orderId);
    reset();
  }, [orderId, reset]);
  return {
    txStatus,
    txResult,
    doCancel,
    submitTx,
    reset
  };
}

function useCreatePoolForm({
  baseAsset,
  quoteAsset
}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = reactUtils.useAsyncProcess();
  const [txResult, setTxResult] = React.useState(null);
  const [baseBalance, baseBalanceStatus] = reactHub.useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = reactHub.useAssetBalance(quoteAsset);
  const balanceStatus = reactUtils.useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);
  const {
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePricePerBase,
    inversePricePerBaseStatus,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    reset: resetPriceInputs
  } = usePriceInputs({ baseAsset, quoteAsset, updateOtherAsset: false });
  const setAmountBasePercentOfBalance = React.useCallback((percentage) => {
    const bnBalance = new BigNumber__default["default"](baseBalance.amount.toString()).times(percentage / 100);
    setAmountBaseFixed(bnBalance.toFixed(0));
  }, [baseBalance.amount, setAmountBaseFixed]);
  const setAmountQuotePercentOfBalance = React.useCallback((percentage) => {
    const bnBalance = new BigNumber__default["default"](quoteBalance.amount.toString()).times(percentage / 100);
    setAmountQuoteFixed(bnBalance.toFixed(0));
  }, [quoteBalance.amount, setAmountQuoteFixed]);
  const [amountQuote, amountQuoteStatus] = reactHub.useDetailedBalance(quoteAsset, amountQuoteFixed);
  const [amountBase, amountBaseStatus] = reactHub.useDetailedBalance(baseAsset, amountBaseFixed);
  const loadingStatus = reactUtils.useAsyncStatuses([balanceStatus]);
  const inputParams = React.useMemo(() => {
    return {
      coinA: baseAsset.id,
      coinB: quoteAsset.id,
      coinAmountA: amountBaseFixed,
      coinAmountB: amountQuoteFixed
    };
  }, [baseAsset.id, quoteAsset.id, amountBaseFixed, amountQuoteFixed]);
  const validationErrors = React.useMemo(() => {
    const errors = {};
    const bnPrice = new BigNumber__default["default"](price);
    const bnAmountA = new BigNumber__default["default"](inputParams.coinAmountA);
    const bnAmountB = new BigNumber__default["default"](inputParams.coinAmountB);
    if (!inputParams.coinA)
      errors.baseAsset = "Base asset must be set";
    if (!inputParams.coinB)
      errors.quoteAsset = "Quote asset must be set";
    if (bnPrice.isNaN() || bnPrice.lte(0)) {
      errors.price = "Must be a number > 0";
    }
    if (bnAmountA.isNaN() || bnAmountA.lte(0)) {
      errors.amountBaseFixed = "Amount must be a number > 0";
    }
    if (bnAmountB.isNaN() || bnAmountB.lte(0)) {
      errors.amountQuoteFixed = "Amount must be a number > 0";
    }
    if (bnAmountA.gt(baseBalance.amount.toString())) {
      errors.amountBaseFixed = "Exceeds available balance";
    }
    if (bnAmountB.gt(quoteBalance.amount.toString())) {
      errors.amountQuoteFixed = "Exceeds available balance";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [price, inputParams, baseBalance.amount, quoteBalance.amount]);
  const canSubmit = !validationErrors && !loadingStatus.pending;
  const submitTx = React.useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.createPool(inputParams))) != null ? _a : null);
    });
  }, [onex, inputParams, canSubmit, resetTxStatus, attemptTx]);
  const reset = React.useCallback(() => {
    resetTxStatus();
    setTxResult(null);
    resetPriceInputs();
  }, [resetPriceInputs, resetTxStatus]);
  React.useEffect(reset, [baseAsset.id, quoteAsset.id, reset]);
  return {
    txStatus,
    txResult,
    loadingStatus,
    balanceStatus,
    pricePerBaseStatus,
    inversePricePerBaseStatus,
    amountQuoteStatus,
    amountBaseStatus,
    baseAsset,
    quoteAsset,
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,
    validationErrors,
    canSubmit,
    baseBalance,
    quoteBalance,
    pricePerBase,
    pricePerQuote: inversePricePerBase,
    amountQuote,
    amountBase,
    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,
    setAmountBasePercentOfBalance,
    setAmountQuotePercentOfBalance,
    submitTx,
    reset
  };
}

function useProposalCreateState() {
  const form = reactHub.useSubmitProposalForm();
  const [amountDisplayValue, setAmountDisplayValue] = reactHub.useFixedAmountInput({
    value: form.amountFixed,
    onChangeValue: form.setAmountFixed,
    meta: form.amount.asset.meta
  });
  return {
    form: {
      ...form,
      amountDisplayValue,
      setAmountDisplayValue
    }
  };
}
const ProposalCreateContext = React.createContext(null);
function useProposalCreate() {
  const state = React.useContext(ProposalCreateContext);
  if (!state)
    throw new Error("ProposalCreateContext missing");
  return state;
}
function ProposalCreateProvider({ children }) {
  const state = useProposalCreateState();
  return /* @__PURE__ */ React__default["default"].createElement(ProposalCreateContext.Provider, {
    value: state
  }, children);
}

function useGovernanceState() {
  const [allProposals, loadingStatus] = reactHub.useProposals();
  const activeProposals = React.useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_VOTING_PERIOD";
    });
  }, [allProposals]);
  const passedProposals = React.useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_PASSED";
    });
  }, [allProposals]);
  const rejectedProposals = React.useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_REJECTED" || proposal.status === "PROPOSAL_STATUS_FAILED";
    });
  }, [allProposals]);
  const failedProposals = React.useMemo(() => {
    return allProposals.filter((proposal) => {
      return proposal.status === "PROPOSAL_STATUS_FAILED";
    });
  }, [allProposals]);
  return {
    governanceRoute: "/governance",
    allProposals,
    activeProposals,
    passedProposals,
    rejectedProposals,
    failedProposals,
    loadingStatus
  };
}
const GovernanceContext = React.createContext(null);
function useGovernance() {
  const state = React.useContext(GovernanceContext);
  if (!state)
    throw new Error("GovernanceContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$5() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading governance"
  });
}
function DEFAULT_ERROR_RENDER_FN$5(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function GovernanceProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$5,
  renderError = DEFAULT_ERROR_RENDER_FN$5
}) {
  const state = useGovernanceState();
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(GovernanceContext.Provider, {
    value: state
  }, children);
}

function useClaimRewardState(validatorAddressStr, { redelegate = false } = {}) {
  const form = reactHub.useClaimRewardForm({ validatorAddressStr, redelegate });
  return form;
}
const ClaimRewardContext = React.createContext(null);
function useClaimReward() {
  const state = React.useContext(ClaimRewardContext);
  if (!state)
    throw new Error("ClaimRewardContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$4() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading transfer form"
  });
}
function DEFAULT_ERROR_RENDER_FN$4(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ClaimRewardProvider({
  children,
  validatorId,
  redelegate,
  renderLoading = DEFAULT_LOADING_RENDER_FN$4,
  renderError = DEFAULT_ERROR_RENDER_FN$4
}) {
  const state = useClaimRewardState(validatorId, { redelegate });
  if (state.loadingStatus.pending || !state.delegation)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(ClaimRewardContext.Provider, {
    value: state
  }, children);
}

const PROPOSAL_STATUS = {
  PROPOSAL_STATUS_UNSPECIFIED: "Unspecified",
  PROPOSAL_STATUS_DEPOSIT_PERIOD: "Deposit Period",
  PROPOSAL_STATUS_VOTING_PERIOD: "Voting Period",
  PROPOSAL_STATUS_PASSED: "Passed",
  PROPOSAL_STATUS_REJECTED: "Rejected",
  PROPOSAL_STATUS_FAILED: "Failed",
  UNRECOGNIZED: "Unrecognized"
};
const PROPOSAL_TYPE = {
  ParameterChangeProposal: {
    name: "Parameter Change"
  },
  FundAccountProposal: {
    name: "Fund Account"
  },
  SoftwareUpgradeProposal: {
    name: "Software Upgrade"
  }
};
function isUndefined(index) {
  return typeof index === "undefined";
}
function getProposalType(index) {
  var _a;
  const undefinedType = {
    name: "Unknown"
  };
  return isUndefined(index) ? undefinedType : (_a = PROPOSAL_TYPE[index]) != null ? _a : {
    name: index == null ? void 0 : index.split("/").pop()
  };
}
function getProposalStatus(index) {
  return isUndefined(index) ? "Unknown" : PROPOSAL_STATUS[index];
}
function renderRemainingDays(endDate) {
  if (!endDate)
    return "No end date specified";
  const timeDifference = (Date.parse(endDate.toString()) - Date.now()) / 1e3 / 60 / 60 / 24;
  if (timeDifference < 0)
    return "Voting Ended";
  if (timeDifference < 1)
    return "Less than 1 day left";
  if (timeDifference < 2)
    return "1 day left";
  return `${Math.floor(timeDifference)} days left`;
}
function calculateVotesChartData(proposal) {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i;
  return ((_a = proposal == null ? void 0 : proposal.results) == null ? void 0 : _a.total) ? [
    new BigNumber__default["default"]((_c = (_b = proposal.results) == null ? void 0 : _b.yes) != null ? _c : 0).dividedBy(proposal.results.total).times(100).toNumber(),
    new BigNumber__default["default"]((_e = (_d = proposal.results) == null ? void 0 : _d.no) != null ? _e : 0).plus((_g = (_f = proposal.results) == null ? void 0 : _f.noWithVeto) != null ? _g : 0).dividedBy(proposal.results.total).times(100).toNumber(),
    new BigNumber__default["default"]((_i = (_h = proposal.results) == null ? void 0 : _h.abstain) != null ? _i : 0).dividedBy(proposal.results.total).times(100).toNumber()
  ] : [0, 0, 0];
}

const governanceUtils = /*#__PURE__*/Object.freeze({
  __proto__: null,
  PROPOSAL_STATUS: PROPOSAL_STATUS,
  PROPOSAL_TYPE: PROPOSAL_TYPE,
  isUndefined: isUndefined,
  getProposalType: getProposalType,
  getProposalStatus: getProposalStatus,
  renderRemainingDays: renderRemainingDays,
  calculateVotesChartData: calculateVotesChartData
});

function useProposalDetailState(proposalId) {
  const [proposal, loadingStatus] = reactHub.useProposal(proposalId);
  const votesChartData = React.useMemo(() => {
    return proposal ? calculateVotesChartData(proposal) : [0, 0, 0];
  }, [proposal]);
  const isVotingPeriod = (proposal == null ? void 0 : proposal.status) === "PROPOSAL_STATUS_VOTING_PERIOD";
  return {
    proposalId,
    proposal,
    votesChartData,
    isVotingPeriod,
    loadingStatus
  };
}
const ProposalDetailContext = React.createContext(null);
function useProposalDetail() {
  const state = React.useContext(ProposalDetailContext);
  if (!state)
    throw new Error("ProposalDetailContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$3() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading proposal"
  });
}
function DEFAULT_ERROR_RENDER_FN$3(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ProposalDetailProvider({
  children,
  getProposalId,
  renderLoading = DEFAULT_LOADING_RENDER_FN$3,
  renderError = DEFAULT_ERROR_RENDER_FN$3
}) {
  const state = useProposalDetailState(getProposalId());
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  if (!state.proposal)
    return renderLoading();
  return /* @__PURE__ */ React__default["default"].createElement(ProposalDetailContext.Provider, {
    value: state
  }, children);
}

function useValidatorDetailState(validatorId) {
  const [delegation, delegationStatus] = reactHub.useDelegation({
    chain: hub.OnomyHubConstants.ONOMY_CHAIN,
    id: validatorId
  });
  const claimRewards = reactHub.useClaimRewardForm({ validatorAddressStr: validatorId });
  const loadingStatus = reactUtils.useAsyncStatuses([delegationStatus, claimRewards.loadingStatus]);
  const validatorDetailRoute = `/wallet/staking/${validatorId}`;
  return {
    delegation,
    claimRewards,
    loadingStatus,
    validatorDetailRoute
  };
}
const ValidatorDetailContext = React.createContext(null);
function useValidatorDetail() {
  const state = React.useContext(ValidatorDetailContext);
  if (!state)
    throw new Error("ValidatorDetailContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN$2() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading validator"
  });
}
function DEFAULT_ERROR_RENDER_FN$2(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function ValidatorDetailProvider({
  children,
  getValidatorId,
  renderLoading = DEFAULT_LOADING_RENDER_FN$2,
  renderError = DEFAULT_ERROR_RENDER_FN$2
}) {
  const state = useValidatorDetailState(getValidatorId());
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  if (!state.delegation)
    return renderLoading();
  return /* @__PURE__ */ React__default["default"].createElement(ValidatorDetailContext.Provider, {
    value: state
  }, children);
}

function useRedeemDropForm({ dropId = "" } = {}) {
  const onex = useOnexClient();
  const [attemptTx, txStatus, resetTxStatus] = reactUtils.useAsyncProcess();
  const [txResult, setTxResult] = React.useState(null);
  const validationErrors = React.useMemo(() => {
    const errors = {};
    const bnDropId = new BigNumber__default["default"](dropId);
    if (bnDropId.isNaN() || bnDropId.lte(0)) {
      errors.dropId = "Coin A amount must be a number > 0";
    }
    return Object.keys(errors).length > 0 ? errors : null;
  }, [dropId]);
  const canSubmit = !validationErrors;
  const doRedeem = React.useCallback((id) => {
    resetTxStatus();
    setTxResult(null);
    return attemptTx(async () => {
      var _a;
      return setTxResult((_a = await (onex == null ? void 0 : onex.redeemDrop({ uid: id != null ? id : dropId }))) != null ? _a : null);
    });
  }, [onex, resetTxStatus, attemptTx, dropId]);
  const submitTx = React.useCallback((e) => {
    e == null ? void 0 : e.preventDefault();
    if (!canSubmit)
      return Promise.resolve();
    return doRedeem(dropId);
  }, [canSubmit, dropId, doRedeem]);
  const reset = React.useCallback(() => {
    resetTxStatus();
    setTxResult(null);
  }, [resetTxStatus]);
  React.useEffect(reset, [dropId, reset]);
  return {
    txStatus,
    txResult,
    canSubmit,
    validationErrors,
    dropId,
    doRedeem,
    submitTx,
    reset
  };
}

function getAssetKey(asset) {
  return [asset.chain.name, asset.id].join("|");
}
function useWalletBridgeFormState() {
  const { hub: hub$1 } = reactHub.useOnomyHub();
  if (!hub$1)
    throw new Error("onomy hub not present");
  const bridgeAssetMap = React.useMemo(() => {
    return hub$1.bridges.reduce((res, bridge) => {
      const bridgeFromAsset = bridge.getFromAsset();
      const bridgeToAsset = bridge.getToAsset();
      const fromAssetKey = getAssetKey(bridgeFromAsset);
      const toAssetKey = getAssetKey(bridgeToAsset);
      const fromAssetRecord = res[fromAssetKey] = res[fromAssetKey] || {
        fromAsset: bridgeFromAsset,
        toAssets: {}
      };
      fromAssetRecord.toAssets[toAssetKey] = bridgeToAsset;
      return res;
    }, {});
  }, [hub$1.bridges]);
  const fromAssets = React.useMemo(() => Object.values(bridgeAssetMap).map((v) => v.fromAsset), [bridgeAssetMap]);
  const [fromAsset, setFromAsset] = React.useState(hub.OnomyHubConstants.NOM_ASSET);
  const toAssets = React.useMemo(() => {
    var _a, _b;
    const key = getAssetKey(fromAsset);
    return Object.values((_b = (_a = bridgeAssetMap[key]) == null ? void 0 : _a.toAssets) != null ? _b : {});
  }, [fromAsset, bridgeAssetMap]);
  const [toAsset, setToAsset] = React.useState(hub.OnomyHubConstants.ETH_NOM_ASSET);
  const form = reactHub.useBridgeForm({
    fromAsset,
    toAsset
  });
  const [amountDisplayValue, setAmountDisplayValue] = reactHub.useFixedAmountInput({
    value: form.bridgeAmountFixed,
    onChangeValue: form.setBridgeAmountFixed,
    meta: form.fromAsset.meta
  });
  return {
    ...form,
    fromAssets,
    toAssets,
    amountDisplayValue,
    setAmountDisplayValue,
    setFromAsset,
    setToAsset
  };
}
const WalletBridgeFormContext = React.createContext(null);
function DEFAULT_LOADING_RENDER_FN$1() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading bridge form"
  });
}
function DEFAULT_ERROR_RENDER_FN$1(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function WalletBridgeFormProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN$1,
  renderError = DEFAULT_ERROR_RENDER_FN$1
}) {
  const state = useWalletBridgeFormState();
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(WalletBridgeFormContext.Provider, {
    value: state
  }, children);
}

function useTransferAssetState(asset) {
  const form = reactHub.useTransferAssetForm(asset);
  return form;
}
const TransferAssetContext = React.createContext(null);
function useTransferAsset() {
  const state = React.useContext(TransferAssetContext);
  if (!state)
    throw new Error("TransferAssetContext missing");
  return state;
}
function DEFAULT_LOADING_RENDER_FN() {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderLoadingIndicator, {
    text: "Loading transfer form"
  });
}
function DEFAULT_ERROR_RENDER_FN(error) {
  return /* @__PURE__ */ React__default["default"].createElement(PlaceholderErrorDisplay, {
    error
  });
}
function TransferAssetProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  asset = null
}) {
  const state = useTransferAssetState(asset);
  if (state.loadingStatus.pending)
    return renderLoading();
  if (state.loadingStatus.error)
    return renderError(state.loadingStatus.error);
  return /* @__PURE__ */ React__default["default"].createElement(TransferAssetContext.Provider, {
    value: state
  }, children);
}

Object.defineProperty(exports, 'CosmosClient', {
  enumerable: true,
  get: function () { return client.CosmosClient; }
});
Object.defineProperty(exports, 'OnexClient', {
  enumerable: true,
  get: function () { return client.OnexClient; }
});
Object.defineProperty(exports, 'OnomyClient', {
  enumerable: true,
  get: function () { return client.OnomyClient; }
});
Object.defineProperty(exports, 'useAsyncProcess', {
  enumerable: true,
  get: function () { return reactUtils.useAsyncProcess; }
});
Object.defineProperty(exports, 'useAsyncStatuses', {
  enumerable: true,
  get: function () { return reactUtils.useAsyncStatuses; }
});
Object.defineProperty(exports, 'OnomyHubConstants', {
  enumerable: true,
  get: function () { return hub.OnomyHubConstants; }
});
Object.defineProperty(exports, 'AmountDisplay', {
  enumerable: true,
  get: function () { return reactHub.AmountDisplay; }
});
Object.defineProperty(exports, 'BigNumDisplay', {
  enumerable: true,
  get: function () { return reactHub.BigNumDisplay; }
});
Object.defineProperty(exports, 'FiatValueDisplay', {
  enumerable: true,
  get: function () { return reactHub.FiatValueDisplay; }
});
Object.defineProperty(exports, 'RatioAsPercentage', {
  enumerable: true,
  get: function () { return reactHub.RatioAsPercentage; }
});
Object.defineProperty(exports, 'TruncatedAddress', {
  enumerable: true,
  get: function () { return reactHub.TruncatedAddress; }
});
Object.defineProperty(exports, 'serializeAsset', {
  enumerable: true,
  get: function () { return reactHub.serializeAsset; }
});
Object.defineProperty(exports, 'serializeBalance', {
  enumerable: true,
  get: function () { return reactHub.serializeBalance; }
});
Object.defineProperty(exports, 'serializeBalances', {
  enumerable: true,
  get: function () { return reactHub.serializeBalances; }
});
Object.defineProperty(exports, 'useAddress', {
  enumerable: true,
  get: function () { return reactHub.useAddress; }
});
Object.defineProperty(exports, 'useAssetBalance', {
  enumerable: true,
  get: function () { return reactHub.useAssetBalance; }
});
Object.defineProperty(exports, 'useClaimRewardForm', {
  enumerable: true,
  get: function () { return reactHub.useClaimRewardForm; }
});
Object.defineProperty(exports, 'useConnectedWallet', {
  enumerable: true,
  get: function () { return reactHub.useConnectedWallet; }
});
Object.defineProperty(exports, 'useDelegateAssetForm', {
  enumerable: true,
  get: function () { return reactHub.useDelegateAssetForm; }
});
Object.defineProperty(exports, 'useDelegations', {
  enumerable: true,
  get: function () { return reactHub.useDelegations; }
});
Object.defineProperty(exports, 'useFixedAmountInput', {
  enumerable: true,
  get: function () { return reactHub.useFixedAmountInput; }
});
Object.defineProperty(exports, 'useGovVoteForm', {
  enumerable: true,
  get: function () { return reactHub.useGovVoteForm; }
});
Object.defineProperty(exports, 'useHoldings', {
  enumerable: true,
  get: function () { return reactHub.useHoldings; }
});
Object.defineProperty(exports, 'useIsAddressValid', {
  enumerable: true,
  get: function () { return reactHub.useIsAddressValid; }
});
Object.defineProperty(exports, 'useMoveDelegationForm', {
  enumerable: true,
  get: function () { return reactHub.useMoveDelegationForm; }
});
Object.defineProperty(exports, 'useOnomyHub', {
  enumerable: true,
  get: function () { return reactHub.useOnomyHub; }
});
Object.defineProperty(exports, 'useTransferAssetForm', {
  enumerable: true,
  get: function () { return reactHub.useTransferAssetForm; }
});
Object.defineProperty(exports, 'useWalletsForChain', {
  enumerable: true,
  get: function () { return reactHub.useWalletsForChain; }
});
exports.ClaimRewardProvider = ClaimRewardProvider;
exports.ExchangePairProvider = ExchangePairProvider;
exports.GovernanceProvider = GovernanceProvider;
exports.LiquidityPairProvider = LiquidityPairProvider;
exports.OnomyCoreProvider = OnomyCoreProvider;
exports.OnomyDashboardProvider = OnomyDashboardProvider;
exports.ProposalCreateProvider = ProposalCreateProvider;
exports.ProposalDetailProvider = ProposalDetailProvider;
exports.TransferAssetProvider = TransferAssetProvider;
exports.ValidatorDetailProvider = ValidatorDetailProvider;
exports.WalletBridgeFormProvider = WalletBridgeFormProvider;
exports.governanceUtils = governanceUtils;
exports.setOnomyEcosystemConfig = setOnomyEcosystemConfig;
exports.useAddDropsForm = useAddDropsForm;
exports.useCancelOrderForm = useCancelOrderForm;
exports.useClaimReward = useClaimReward;
exports.useCreatePoolForm = useCreatePoolForm;
exports.useExchangePair = useExchangePair;
exports.useGovernance = useGovernance;
exports.useIndexedOrderBookForm = useIndexedOrderBookForm;
exports.useLiquidityPair = useLiquidityPair;
exports.useOnexDayTradeFrame = useOnexDayTradeFrame;
exports.useOnexTradingViewData = useOnexTradingViewData;
exports.useOnomyApp = useOnomyApp;
exports.useOnomyDashboard = useOnomyDashboard;
exports.useProposalCreate = useProposalCreate;
exports.useProposalDetail = useProposalDetail;
exports.useRedeemDropForm = useRedeemDropForm;
exports.useTransferAsset = useTransferAsset;
exports.useValidatorDetail = useValidatorDetail;
