import React, { createContext, useContext } from 'react';
import { useClaimRewardForm } from '@onomy/react-hub';
import { PlaceholderLoadingIndicator } from './components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from './components/PlaceholderErrorDisplay';

function useClaimRewardState(validatorAddressStr: string, { redelegate = false } = {}) {
  const form = useClaimRewardForm({ validatorAddressStr, redelegate });
  return form;
}

export type ClaimRewardContextType = ReturnType<typeof useClaimRewardState>;

const ClaimRewardContext = createContext<ClaimRewardContextType>(null as any);

export function useClaimReward() {
  const state = useContext(ClaimRewardContext);
  if (!state) throw new Error('ClaimRewardContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading transfer form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ClaimRewardProvider({
  children,
  validatorId,
  redelegate,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  validatorId: string;
  redelegate?: boolean;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useClaimRewardState(validatorId, { redelegate });

  if (state.loadingStatus.pending || !state.delegation) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return <ClaimRewardContext.Provider value={state}>{children}</ClaimRewardContext.Provider>;
}
