import React, { createContext, useContext } from 'react';
import { useFixedAmountInput, useSubmitProposalForm } from '@onomy/react-hub';

function useProposalCreateState() {
  const form = useSubmitProposalForm();

  const [amountDisplayValue, setAmountDisplayValue] = useFixedAmountInput({
    value: form.amountFixed,
    onChangeValue: form.setAmountFixed,
    meta: form.amount.asset.meta,
  });

  return {
    form: {
      ...form,
      amountDisplayValue,
      setAmountDisplayValue,
    },
  };
}

export type ProposalCreateContextType = ReturnType<typeof useProposalCreateState>;

const ProposalCreateContext = createContext<ProposalCreateContextType>(null as any);

export function useProposalCreate() {
  const state = useContext(ProposalCreateContext);
  if (!state) throw new Error('ProposalCreateContext missing');
  return state;
}

export function ProposalCreateProvider({ children }: { children: JSX.Element | JSX.Element[] }) {
  const state = useProposalCreateState();

  return <ProposalCreateContext.Provider value={state}>{children}</ProposalCreateContext.Provider>;
}
